@@ -2,7 +2,7 @@
  * access.c : CD digital audio input module for vlc using libcdio
  *****************************************************************************
  * Copyright (C) 2000, 2003, 2004, 2005 the VideoLAN team
- * $Id: access.c 11664 2005-07-09 06:17:09Z courmisch $
+ * $Id: access.c 17183 2006-10-20 21:33:16Z hartman $
  *
  * Authors: Rocky Bernstein <rocky@panix.com>
  *          Laurent Aimar <fenrir@via.ecp.fr>
@@ -20,7 +20,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -100,7 +100,7 @@ cdio_log_handler (cdio_log_level_t level, const char message[])
     break;
   default:
     msg_Warn( p_cdda_input, message,
-            "The above message had unknown cdio log level",
+            "the above message had unknown cdio log level",
             level);
   }
   return;
@@ -284,7 +284,7 @@ static block_t * CDDAReadBlocks( access_t * p_access )
     p_block = block_New( p_access, i_blocks * CDIO_CD_FRAMESIZE_RAW );
     if( !p_block)
     {
-      msg_Err( p_access, "Cannot get a new block of size: %i",
+      msg_Err( p_access, "cannot get a new block of size: %i",
                i_blocks * CDIO_CD_FRAMESIZE_RAW );
       return NULL;
     }
@@ -495,7 +495,7 @@ CDDASeek( access_t * p_access, int64_t i_pos )
     }
     
     p_access->info.i_pos = i_pos;
-
+    p_access->info.b_eof = VLC_FALSE;
     return VLC_SUCCESS;
 }
 
@@ -679,7 +679,7 @@ CDDAOpen( vlc_object_t *p_this )
          || p_cdda->i_blocks_per_read > MAX_BLOCKS_PER_READ )
     {
         msg_Warn( p_cdda_input,
-                "Number of blocks (%d) has to be between %d and %d. "
+                "number of blocks (%d) has to be between %d and %d. "
                 "Using %d.",
                 p_cdda->i_blocks_per_read,
                 MIN_BLOCKS_PER_READ, MAX_BLOCKS_PER_READ,
@@ -753,7 +753,7 @@ CDDAOpen( vlc_object_t *p_this )
 	  cdio_cddap_verbose_set(p_cdda->paranoia_cd, CDDA_MESSAGE_PRINTIT, 
 				 CDDA_MESSAGE_PRINTIT);
 	  if ( 0 != cdio_cddap_open(p_cdda->paranoia_cd) ) {
-	    msg_Warn( p_cdda_input, "Unable to get paranoia support - "
+	    msg_Warn( p_cdda_input, "unable to get paranoia support - "
 		      "continuing without it." );
 	    p_cdda->e_paranoia = paranoia_none;
 	  } else {
