@@ -2,7 +2,7 @@
  * ffmpeg.h: decoder using the ffmpeg library
  *****************************************************************************
  * Copyright (C) 2001 the VideoLAN team
- * $Id: ffmpeg.h 11867 2005-07-27 16:55:17Z massiot $
+ * $Id: ffmpeg.h 16798 2006-09-22 23:56:12Z xtophe $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 #include "codecs.h"                                      /* BITMAPINFOHEADER */
@@ -68,8 +68,13 @@ void E_(CloseAudioEncoder)( vlc_object_t * );
 int  E_(OpenDemux) ( vlc_object_t * );
 void E_(CloseDemux)( vlc_object_t * );
 
+/* Mux module */
+int  E_(OpenMux) ( vlc_object_t * );
+void E_(CloseMux)( vlc_object_t * );
+
 /* Video filter module */
 int  E_(OpenFilter)( vlc_object_t * );
+int  E_(OpenCropPadd)( vlc_object_t * );
 void E_(CloseFilter)( vlc_object_t * );
 int  E_(OpenDeinterlace)( vlc_object_t * );
 void E_(CloseDeinterlace)( vlc_object_t * );
@@ -84,28 +89,32 @@ void E_(ClosePostproc)( decoder_t *, void * );
  * Module descriptor help strings
  *****************************************************************************/
 #define DR_TEXT N_("Direct rendering")
+/* FIXME Does somebody who knows what it does, explain */
+#define DR_LONGTEXT N_("Direct rendering")
 
 #define ERROR_TEXT N_("Error resilience")
 #define ERROR_LONGTEXT N_( \
-    "ffmpeg can do error resilience.\n" \
+    "Ffmpeg can do error resilience.\n" \
     "However, with a buggy encoder (such as the ISO MPEG-4 encoder from M$) " \
     "this can produce a lot of errors.\n" \
     "Valid values range from 0 to 4 (0 disables all errors resilience).")
 
 #define BUGS_TEXT N_("Workaround bugs")
 #define BUGS_LONGTEXT N_( \
-    "Try to fix some bugs\n" \
+    "Try to fix some bugs:\n" \
     "1  autodetect\n" \
     "2  old msmpeg4\n" \
     "4  xvid interlaced\n" \
     "8  ump4 \n" \
     "16 no padding\n" \
     "32 ac vlc\n" \
-    "64 Qpel chroma")
+    "64 Qpel chroma.\n" \
+    "This must be the sum of the values. For example, to fix \"ac vlc\" and " \
+    "\"ump4\", enter 40.")
 
 #define HURRYUP_TEXT N_("Hurry up")
 #define HURRYUP_LONGTEXT N_( \
-    "Allow the decoder to partially decode or skip frame(s) " \
+    "The decoder can partially decode or skip frame(s) " \
     "when there is not enough time. It's useful with low CPU power " \
     "but it can produce distorted pictures.")
 
@@ -118,20 +127,29 @@ void E_(ClosePostproc)( decoder_t *, void * );
 #define DEBUG_TEXT N_( "Debug mask" )
 #define DEBUG_LONGTEXT N_( "Set ffmpeg debug mask" )
 
+/* TODO: Use a predefined list, with 0,1,2,4,7 */
 #define VISMV_TEXT N_( "Visualize motion vectors" )
-#define VISMV_LONGTEXT N_( "Set motion vectors visualization mask.\n" \
+#define VISMV_LONGTEXT N_( \
+    "You can overlay the motion vectors (arrows showing how the images move) "\
+    "on the image. This value is a mask, based on these values:\n"\
     "1 - visualize forward predicted MVs of P frames\n" \
     "2 - visualize forward predicted MVs of B frames\n" \
-    "4 - visualize backward predicted MVs of B frames" )
+    "4 - visualize backward predicted MVs of B frames\n" \
+    "To visualize all vectors, the value should be 7." )
 
 #define LOWRES_TEXT N_( "Low resolution decoding" )
-#define LOWRES_LONGTEXT N_( "Will only decode a low resolution version of " \
-    "the video." )
+#define LOWRES_LONGTEXT N_( "Only decode a low resolution version of " \
+    "the video. This requires less processing power" )
 
-#define LIBAVCODEC_PP_TEXT N_("ffmpeg post processing filter chains")
+#define SKIPLOOPF_TEXT N_( "Skip the loop filter for H.264 decoding" )
+#define SKIPLOOPF_LONGTEXT N_( "Skipping the loop filter (aka deblocking) " \
+    "usually has a detrimental effect on quality. However it provides a big " \
+    "speedup for high definition streams." )
+
+#define LIBAVCODEC_PP_TEXT N_("FFmpeg post processing filter chains")
 /* FIXME (cut/past from ffmpeg */
 #define LIBAVCODEC_PP_LONGTEXT \
-"<filterName>[:<option>[:<option>...]][[,|/][-]<filterName>[:<option>...]]...\n" \
+N_("<filterName>[:<option>[:<option>...]][[,|/][-]<filterName>[:<option>...]]...\n" \
 "long form example:\n" \
 "vdeblock:autoq/hdeblock:autoq/linblenddeint    default,-vdeblock\n" \
 "short form example:\n" \
@@ -163,7 +181,7 @@ void E_(ClosePostproc)( decoder_t *, void * );
 "fa     fast                                    h1:a,v1:a,dr:a,al\n" \
 "tn     tmpnoise        (3 Thresholds)          Temporal Noise Reducer\n" \
 "                       1. <= 2. <= 3.          larger -> stronger filtering\n" \
-"fq     forceQuant      <quantizer>             Force quantizer\n"
+"fq     forceQuant      <quantizer>             Force quantizer\n")
 
 /*
  * Encoder options
@@ -171,103 +189,103 @@ void E_(ClosePostproc)( decoder_t *, void * );
 #define ENC_CFG_PREFIX "sout-ffmpeg-"
 
 #define ENC_KEYINT_TEXT N_( "Ratio of key frames" )
-#define ENC_KEYINT_LONGTEXT N_( "Allows you to specify the number of frames " \
+#define ENC_KEYINT_LONGTEXT N_( "Number of frames " \
   "that will be coded for one key frame." )
 
 #define ENC_BFRAMES_TEXT N_( "Ratio of B frames" )
-#define ENC_BFRAMES_LONGTEXT N_( "Allows you to specify the number of " \
+#define ENC_BFRAMES_LONGTEXT N_( "Number of " \
   "B frames that will be coded between two reference frames." )
 
 #define ENC_VT_TEXT N_( "Video bitrate tolerance" )
-#define ENC_VT_LONGTEXT N_( "Allows you to specify the video bitrate " \
-  "tolerance in kbit/s." )
+#define ENC_VT_LONGTEXT N_( "Video bitrate tolerance in kbit/s." )
 
-#define ENC_INTERLACE_TEXT N_( "Enable interlaced encoding" )
-#define ENC_INTERLACE_LONGTEXT N_( "Allows you to enable dedicated " \
+#define ENC_INTERLACE_TEXT N_( "Interlaced encoding" )
+#define ENC_INTERLACE_LONGTEXT N_( "Enable dedicated " \
   "algorithms for interlaced frames." )
 
-#define ENC_INTERLACE_ME_TEXT N_( "Enable interlaced motion estimation" )
-#define ENC_INTERLACE_ME_LONGTEXT N_( "Allows you to enable interlaced " \
-  "motion estimation algorithms. It requires more CPU." )
+#define ENC_INTERLACE_ME_TEXT N_( "Interlaced motion estimation" )
+#define ENC_INTERLACE_ME_LONGTEXT N_( "Enable interlaced " \
+  "motion estimation algorithms. This requires more CPU." )
 
-#define ENC_PRE_ME_TEXT N_( "Enable pre motion estimation" )
-#define ENC_PRE_ME_LONGTEXT N_( "Allows you to enable the pre motion " \
-  "estimation." )
+#define ENC_PRE_ME_TEXT N_( "Pre-motion estimation" )
+#define ENC_PRE_ME_LONGTEXT N_( "Enable the pre-motion " \
+  "estimation algorithm.")
 
-#define ENC_RC_STRICT_TEXT N_( "Enable strict rate control" )
-#define ENC_RC_STRICT_LONGTEXT N_( "Allows you to enable the strict rate " \
+#define ENC_RC_STRICT_TEXT N_( "Strict rate control" )
+#define ENC_RC_STRICT_LONGTEXT N_( "Enable the strict rate " \
   "control algorithm." )
 
 #define ENC_RC_BUF_TEXT N_( "Rate control buffer size" )
-#define ENC_RC_BUF_LONGTEXT N_( "Allows you to specify the rate control " \
-  "buffer size." )
+#define ENC_RC_BUF_LONGTEXT N_( "Rate control " \
+  "buffer size (in kbytes). A bigger buffer will allow for better rate " \
+  "control, but will cause a delay in the stream." )
 
 #define ENC_RC_BUF_AGGR_TEXT N_( "Rate control buffer aggressiveness" )
-#define ENC_RC_BUF_AGGR_LONGTEXT N_( "Allows you to specify the rate control "\
+#define ENC_RC_BUF_AGGR_LONGTEXT N_( "Rate control "\
   "buffer aggressiveness." )
 
 #define ENC_IQUANT_FACTOR_TEXT N_( "I quantization factor" )
-#define ENC_IQUANT_FACTOR_LONGTEXT N_( "Allows you to specify the " \
-  "quantization factor of I frames, compared with P frames (for instance " \
+#define ENC_IQUANT_FACTOR_LONGTEXT N_(  \
+  "Quantization factor of I frames, compared with P frames (for instance " \
   "1.0 => same qscale for I and P frames)." )
 
 #define ENC_NOISE_RED_TEXT N_( "Noise reduction" )
-#define ENC_NOISE_RED_LONGTEXT N_( "Allows you to enable a simple noise " \
+#define ENC_NOISE_RED_LONGTEXT N_( "Enable a simple noise " \
   "reduction algorithm to lower the encoding length and bitrate, at the " \
   "expense of lower quality frames." )
 
-#define ENC_MPEG4_MATRIX_TEXT N_( "Enable MPEG4 quantization matrix" )
-#define ENC_MPEG4_MATRIX_LONGTEXT N_( "Allows you to use the MPEG4 " \
-  "quantization matrix for mpeg2 encoding. This generally yields a " \
+#define ENC_MPEG4_MATRIX_TEXT N_( "MPEG4 quantization matrix" )
+#define ENC_MPEG4_MATRIX_LONGTEXT N_( "Use the MPEG4 " \
+  "quantization matrix for MPEG2 encoding. This generally yields a " \
   "better looking picture, while still retaining the compatibility with " \
-  "standard MPEG-2 decoders.")
+  "standard MPEG2 decoders.")
 
 #define ENC_HQ_TEXT N_( "Quality level" )
-#define ENC_HQ_LONGTEXT N_( "Allows you to specify the quality level " \
+#define ENC_HQ_LONGTEXT N_( "Quality level " \
   "for the encoding of motions vectors (this can slow down the encoding " \
   "very much)." )
 
 #define ENC_HURRYUP_TEXT N_( "Hurry up" )
-#define ENC_HURRYUP_LONGTEXT N_( "Allows you to specify if the encoder " \
-  "should make on-the-fly quality tradeoffs if your CPU can't keep up with " \
+#define ENC_HURRYUP_LONGTEXT N_( "The encoder " \
+  "can make on-the-fly quality tradeoffs if your CPU can't keep up with " \
   "the encoding rate. It will disable trellis quantization, then the rate " \
   "distortion of motion vectors (hq), and raise the noise reduction " \
   "threshold to ease the encoder's task." )
 
 #define ENC_QMIN_TEXT N_( "Minimum video quantizer scale" )
-#define ENC_QMIN_LONGTEXT N_( "Allows you to specify the minimum video " \
+#define ENC_QMIN_LONGTEXT N_( "Minimum video " \
   "quantizer scale." )
 
 #define ENC_QMAX_TEXT N_( "Maximum video quantizer scale" )
-#define ENC_QMAX_LONGTEXT N_( "Allows you to specify the maximum video " \
+#define ENC_QMAX_LONGTEXT N_( "Maximum video " \
   "quantizer scale." )
 
-#define ENC_TRELLIS_TEXT N_( "Enable trellis quantization" )
-#define ENC_TRELLIS_LONGTEXT N_( "Allows you to enable trellis " \
+#define ENC_TRELLIS_TEXT N_( "Trellis quantization" )
+#define ENC_TRELLIS_LONGTEXT N_( "Enable trellis " \
   "quantization (rate distortion for block coefficients)." )
 
-#define ENC_QSCALE_TEXT N_( "Use fixed video quantizer scale" )
-#define ENC_QSCALE_LONGTEXT N_( "Allows you to specify a fixed video " \
+#define ENC_QSCALE_TEXT N_( "Fixed quantizer scale" )
+#define ENC_QSCALE_LONGTEXT N_( "A fixed video " \
   "quantizer scale for VBR encoding (accepted values: 0.01 to 255.0)." )
 
 #define ENC_STRICT_TEXT N_( "Strict standard compliance" )
-#define ENC_STRICT_LONGTEXT N_( "Allows you to force a strict standard " \
+#define ENC_STRICT_LONGTEXT N_( "Force a strict standard " \
   "compliance when encoding (accepted values: -1, 0, 1)." )
 
 #define ENC_LUMI_MASKING_TEXT N_( "Luminance masking" )
-#define ENC_LUMI_MASKING_LONGTEXT N_( "Allows you to raise the quantizer for " \
+#define ENC_LUMI_MASKING_LONGTEXT N_( "Raise the quantizer for " \
   "very bright macroblocks (default: 0.0)." )
 
 #define ENC_DARK_MASKING_TEXT N_( "Darkness masking" )
-#define ENC_DARK_MASKING_LONGTEXT N_( "Allows you to raise the quantizer for " \
+#define ENC_DARK_MASKING_LONGTEXT N_( "Raise the quantizer for " \
   "very dark macroblocks (default: 0.0)." )
 
 #define ENC_P_MASKING_TEXT N_( "Motion masking" )
-#define ENC_P_MASKING_LONGTEXT N_( "Allows you to raise the quantizer for " \
+#define ENC_P_MASKING_LONGTEXT N_( "Raise the quantizer for " \
   "macroblocks with a high temporal complexity (default: 0.0)." )
 
 #define ENC_BORDER_MASKING_TEXT N_( "Border masking" )
-#define ENC_BORDER_MASKING_LONGTEXT N_( "Allows you to raise the quantizer " \
+#define ENC_BORDER_MASKING_LONGTEXT N_( "Raise the quantizer " \
   "for macroblocks at the border of the frame (default: 0.0)." )
 
 #define ENC_LUMA_ELIM_TEXT N_( "Luminance elimination" )
