@@ -2,7 +2,7 @@
  * tarkin.c: tarkin decoder module making use of libtarkin.
  *****************************************************************************
  * Copyright (C) 2001-2003 the VideoLAN team
- * $Id: tarkin.c 11664 2005-07-09 06:17:09Z courmisch $
+ * $Id: tarkin.c 15005 2006-03-31 16:28:03Z xtophe $
  *
  * Authors: Gildas Bazin <gbazin@netcourrier.com>
  *
@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -160,7 +160,7 @@ static void *DecodeBlock( decoder_t *p_dec, block_t **pp_block )
         if( tarkin_synthesis_headerin( &p_sys->ti, &p_sys->tc, &oggpacket )
             < 0 )
         {
-            msg_Err( p_dec, "This bitstream does not contain Tarkin "
+            msg_Err( p_dec, "this bitstream does not contain Tarkin "
                      "video data.");
             block_Release( p_block );
             return NULL;
