@@ -2,7 +2,7 @@
  * rc.c : remote control stdin/stdout module for vlc
  *****************************************************************************
  * Copyright (C) 2004 - 2005 the VideoLAN team
- * $Id: rc.c 13127 2005-11-03 20:39:41Z jpsaman $
+ * $Id: rc.c 16984 2006-10-08 12:14:56Z jpsaman $
  *
  * Author: Peter Surda <shurdeek@panorama.sth.ac.at>
  *         Jean-Paul Saman <jpsaman #_at_# m2x _replaceWith#dot_ nl>
@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -39,6 +39,7 @@
 #include <vlc/vout.h>
 #include <vlc_video.h>
 #include <vlc_osd.h>
+#include <vlc_update.h>
 
 #ifdef HAVE_UNISTD_H
 #    include <unistd.h>
@@ -51,6 +52,9 @@
 
 #include "vlc_error.h"
 #include "network.h"
+#include "vlc_url.h"
+
+#include "charset.h"
 
 #if defined(AF_UNIX) && !defined(AF_LOCAL)
 #    define AF_LOCAL AF_UNIX
@@ -91,10 +95,13 @@ static int  Volume       ( vlc_object_t *, char const *,
                            vlc_value_t, vlc_value_t, void * );
 static int  VolumeMove   ( vlc_object_t *, char const *,
                            vlc_value_t, vlc_value_t, void * );
+static int  VideoConfig  ( vlc_object_t *, char const *,
+                           vlc_value_t, vlc_value_t, void * );
 static int  AudioConfig  ( vlc_object_t *, char const *,
                            vlc_value_t, vlc_value_t, void * );
 static int  Menu         ( vlc_object_t *, char const *,
                            vlc_value_t, vlc_value_t, void * );
+static void checkUpdates( intf_thread_t *p_intf, char *psz_arg );
 
 /* Status Callbacks */
 static int TimeOffsetChanged( vlc_object_t *, char const *,
@@ -134,7 +141,7 @@ void __msg_rc( intf_thread_t *p_intf, const char *psz_fmt, ... )
 
     if( p_intf->p_sys->i_socket == -1 )
     {
-        vprintf( psz_fmt, args );
+        utf8_vfprintf( stdout, psz_fmt, args );
         printf( "\r\n" );
     }
     else
@@ -175,7 +182,7 @@ void __msg_rc( intf_thread_t *p_intf, const char *psz_fmt, ... )
 vlc_module_begin();
     set_shortname( _("RC"));
     set_category( CAT_INTERFACE );
-    set_subcategory( SUBCAT_INTERFACE_GENERAL );
+    set_subcategory( SUBCAT_INTERFACE_MAIN );
     set_description( _("Remote control interface") );
     add_bool( "rc-show-pos", 0, NULL, POS_TEXT, POS_LONGTEXT, VLC_TRUE );
 #ifdef HAVE_ISATTY
@@ -277,7 +284,7 @@ static int Activate( vlc_object_t *p_this )
 
         vlc_UrlParse( &url, psz_host, 0 );
 
-        msg_Dbg( p_intf, "base %s port %d", url.psz_host, url.i_port );
+        msg_Dbg( p_intf, "base: %s, port: %d", url.psz_host, url.i_port );
 
         pi_socket = net_ListenTCP(p_this, url.psz_host, url.i_port);
         if( pi_socket == NULL )
@@ -329,7 +336,7 @@ static int Activate( vlc_object_t *p_this )
         vlc_object_release( p_playlist );
     }
 
-    msg_rc( _("Remote control interface initialized, `h' for help") );
+    msg_rc( _("Remote control interface initialized. Type `help' for help.") );
     return VLC_SUCCESS;
 }
 
@@ -350,7 +357,7 @@ static void Deactivate( vlc_object_t *p_this )
 #endif
         free( p_intf->p_sys->psz_unix_path );
     }
-    vlc_mutex_destroy( &p_intf->p_sys->status_lock );    
+    vlc_mutex_destroy( &p_intf->p_sys->status_lock );
     free( p_intf->p_sys );
 }
 
@@ -367,6 +374,8 @@ static void RegisterCallbacks( intf_thread_t *p_intf )
 
     var_Create( p_intf, "add", VLC_VAR_STRING | VLC_VAR_ISCOMMAND );
     var_AddCallback( p_intf, "add", Playlist, NULL );
+    var_Create( p_intf, "enqueue", VLC_VAR_STRING | VLC_VAR_ISCOMMAND );
+    var_AddCallback( p_intf, "enqueue", Playlist, NULL );
     var_Create( p_intf, "playlist", VLC_VAR_VOID | VLC_VAR_ISCOMMAND );
     var_AddCallback( p_intf, "playlist", Playlist, NULL );
     var_Create( p_intf, "play", VLC_VAR_VOID | VLC_VAR_ISCOMMAND );
@@ -459,7 +468,7 @@ static void RegisterCallbacks( intf_thread_t *p_intf )
 
     /* OSD menu commands */
     var_Create( p_intf, "menu", VLC_VAR_STRING | VLC_VAR_ISCOMMAND );
-    var_AddCallback( p_intf, "menu", Menu, NULL ); 
+    var_AddCallback( p_intf, "menu", Menu, NULL );
 
     /* DVD commands */
     var_Create( p_intf, "pause", VLC_VAR_VOID | VLC_VAR_ISCOMMAND );
@@ -490,6 +499,21 @@ static void RegisterCallbacks( intf_thread_t *p_intf )
     var_Create( p_intf, "normal", VLC_VAR_VOID | VLC_VAR_ISCOMMAND );
     var_AddCallback( p_intf, "normal", Input, NULL );
 
+    var_Create( p_intf, "atrack", VLC_VAR_STRING | VLC_VAR_ISCOMMAND );
+    var_AddCallback( p_intf, "atrack", Input, NULL );
+    var_Create( p_intf, "vtrack", VLC_VAR_STRING | VLC_VAR_ISCOMMAND );
+    var_AddCallback( p_intf, "vtrack", Input, NULL );
+    var_Create( p_intf, "strack", VLC_VAR_STRING | VLC_VAR_ISCOMMAND );
+    var_AddCallback( p_intf, "strack", Input, NULL );
+
+    /* video commands */
+    var_Create( p_intf, "vratio", VLC_VAR_STRING | VLC_VAR_ISCOMMAND );
+    var_AddCallback( p_intf, "vratio", VideoConfig, NULL );
+    var_Create( p_intf, "vcrop", VLC_VAR_STRING | VLC_VAR_ISCOMMAND );
+    var_AddCallback( p_intf, "vcrop", VideoConfig, NULL );
+    var_Create( p_intf, "vzoom", VLC_VAR_STRING | VLC_VAR_ISCOMMAND );
+    var_AddCallback( p_intf, "vzoom", VideoConfig, NULL );
+
     /* audio commands */
     var_Create( p_intf, "volume", VLC_VAR_STRING | VLC_VAR_ISCOMMAND );
     var_AddCallback( p_intf, "volume", Volume, NULL );
@@ -538,7 +562,7 @@ static void Run( intf_thread_t *p_intf )
     p_intf->p_sys->hConsoleIn = GetStdHandle(STD_INPUT_HANDLE);
     if( p_intf->p_sys->hConsoleIn == INVALID_HANDLE_VALUE )
     {
-        msg_Err( p_intf, "Couldn't open STD_INPUT_HANDLE" );
+        msg_Err( p_intf, "couldn't find user input handle" );
         p_intf->b_die = VLC_TRUE;
     }
 #endif
@@ -580,7 +604,7 @@ static void Run( intf_thread_t *p_intf )
             {
                 if( !p_input->b_dead || !p_input->b_die )
                 {
-                    msg_rc( STATUS_CHANGE "( New input: %s )", p_input->input.p_item->psz_uri );
+                    msg_rc( STATUS_CHANGE "( new input: %s )", p_input->input.p_item->psz_uri );
                     msg_rc( STATUS_CHANGE "( audio volume: %d )", config_GetInt( p_intf, "volume" ));
                 }
                 var_AddCallback( p_input, "state", StateChanged, p_intf );
@@ -793,6 +817,10 @@ static void Run( intf_thread_t *p_intf )
 
             Help( p_intf, b_longhelp );
         }
+        else if( !strcmp( psz_cmd, "check-updates" ) )
+        {
+            checkUpdates( p_intf, psz_arg );
+        }
         else switch( psz_cmd[0] )
         {
         case 'f':
@@ -837,7 +865,7 @@ static void Run( intf_thread_t *p_intf )
             break;
 
         default:
-            msg_rc(_("unknown command `%s', type `help' for help"), psz_cmd);
+            msg_rc(_("Unknown command `%s'. Type `help' for help."), psz_cmd);
             break;
         }
 
@@ -873,6 +901,7 @@ static void Help( intf_thread_t *p_intf, vlc_bool_t b_longhelp)
     msg_rc(_("+----[ Remote control commands ]"));
     msg_rc(  "| ");
     msg_rc(_("| add XYZ  . . . . . . . . . . add XYZ to playlist"));
+    msg_rc(_("| enqueue XYZ  . . . . . . . queue XYZ to playlist"));
     msg_rc(_("| playlist . . .  show items currently in playlist"));
     msg_rc(_("| play . . . . . . . . . . . . . . . . play stream"));
     msg_rc(_("| stop . . . . . . . . . . . . . . . . stop stream"));
@@ -897,12 +926,22 @@ static void Help( intf_thread_t *p_intf, vlc_bool_t b_longhelp)
     msg_rc(_("| normal . . . . . . . .  normal playing of stream"));
     msg_rc(_("| f [on|off] . . . . . . . . . . toggle fullscreen"));
     msg_rc(_("| info . . .  information about the current stream"));
+    msg_rc(_("| get_time . . seconds elapsed since stream's beginning"));
+    msg_rc(_("| is_playing . .  1 if a stream plays, 0 otherwise"));
+    msg_rc(_("| get_title . . .  the title of the current stream"));
+    msg_rc(_("| get_length . .  the length of the current stream"));
     msg_rc(  "| ");
     msg_rc(_("| volume [X] . . . . . . . .  set/get audio volume"));
     msg_rc(_("| volup [X]  . . . . .  raise audio volume X steps"));
     msg_rc(_("| voldown [X]  . . . .  lower audio volume X steps"));
     msg_rc(_("| adev [X] . . . . . . . . .  set/get audio device"));
     msg_rc(_("| achan [X]. . . . . . . .  set/get audio channels"));
+    msg_rc(_("| atrack [X] . . . . . . . . . set/get audio track"));
+    msg_rc(_("| vtrack [X] . . . . . . . . . set/get video track"));
+    msg_rc(_("| vratio [X]  . . . . . set/get video aspect ratio"));
+    msg_rc(_("| vcrop [X]  . . . . . . . . .  set/get video crop"));
+    msg_rc(_("| vzoom [X]  . . . . . . . . .  set/get video zoom"));
+    msg_rc(_("| strack [X] . . . . . . . set/get subtitles track"));
     msg_rc(_("| menu [on|off|up|down|left|right|select] use menu"));
     msg_rc(  "| ");
 
@@ -925,7 +964,7 @@ static void Help( intf_thread_t *p_intf, vlc_bool_t b_longhelp)
         msg_rc(_("| time-opacity # . . . . . . . . . . . . . opacity"));
         msg_rc(_("| time-size # . . . . . . . . font size, in pixels"));
         msg_rc(  "| ");
-        msg_rc(_("| logo-file STRING . . . the overlay file path/name"));
+        msg_rc(_("| logo-file STRING . . .the overlay file path/name"));
         msg_rc(_("| logo-x X . . . . . . . . . . . .offset from left"));
         msg_rc(_("| logo-y Y . . . . . . . . . . . . offset from top"));
         msg_rc(_("| logo-position #. . . . . . . . relative position"));
@@ -944,6 +983,9 @@ static void Help( intf_thread_t *p_intf, vlc_bool_t b_longhelp)
         msg_rc(_("| mosaic-cols #. . . . . . . . . . .number of cols"));
         msg_rc(_("| mosaic-keep-aspect-ratio {0,1} . . .aspect ratio"));
         msg_rc(  "| ");
+        msg_rc(_("| check-updates [newer] [equal] [older]\n"
+                 "|               [undef] [info] [source] [binary] [plugin]"));
+        msg_rc(  "| ");
     }
     msg_rc(_("| help . . . . . . . . . . . . . this help message"));
     msg_rc(_("| longhelp . . . . . . . . . a longer help message"));
@@ -1057,7 +1099,7 @@ static int Input( vlc_object_t *p_this, char const *psz_cmd,
     if( ( ( val.i_int == PAUSE_S ) || ( val.i_int == PLAYLIST_PAUSED ) ) &&
         ( strcmp( psz_cmd, "pause" ) != 0 ) )
     {
-        msg_rc( _("press menu select or pause to continue") );
+        msg_rc( _("Press menu select or pause to continue.") );
         vlc_object_release( p_input );
         return VLC_EGENERIC;
     }
@@ -1088,14 +1130,14 @@ static int Input( vlc_object_t *p_this, char const *psz_cmd,
     }
     else if ( !strcmp( psz_cmd, "fastforward" ) )
     {
-        val.i_int = config_GetInt( p_intf, "key-jump+3sec" );
+        val.i_int = config_GetInt( p_intf, "key-jump+extrashort" );
         var_Set( p_intf->p_vlc, "key-pressed", val );
         vlc_object_release( p_input );
         return VLC_SUCCESS;
     }
     else if ( !strcmp( psz_cmd, "rewind" ) )
     {
-        val.i_int = config_GetInt( p_intf, "key-jump-3sec" );
+        val.i_int = config_GetInt( p_intf, "key-jump-extrashort" );
         var_Set( p_intf->p_vlc, "key-pressed", val );
         vlc_object_release( p_input );
         return VLC_SUCCESS;
@@ -1141,7 +1183,7 @@ static int Input( vlc_object_t *p_this, char const *psz_cmd,
                 var_Get( p_input, "chapter", &val );
                 var_Change( p_input, "chapter", VLC_VAR_GETCHOICES,
                             &val_list, NULL );
-                msg_rc( "Currently playing chapter %d/%d",
+                msg_rc( "Currently playing chapter %d/%d.",
                         val.i_int, val_list.p_list->i_count );
                 var_Change( p_this, "chapter", VLC_VAR_FREELIST,
                             &val_list, NULL );
@@ -1180,7 +1222,7 @@ static int Input( vlc_object_t *p_this, char const *psz_cmd,
                 var_Get( p_input, "title", &val );
                 var_Change( p_input, "title", VLC_VAR_GETCHOICES,
                             &val_list, NULL );
-                msg_rc( "Currently playing title %d/%d",
+                msg_rc( "Currently playing title %d/%d.",
                         val.i_int, val_list.p_list->i_count );
                 var_Change( p_this, "title", VLC_VAR_FREELIST,
                             &val_list, NULL );
@@ -1200,6 +1242,81 @@ static int Input( vlc_object_t *p_this, char const *psz_cmd,
         vlc_object_release( p_input );
         return VLC_SUCCESS;
     }
+    else if(    !strcmp( psz_cmd, "atrack" )
+             || !strcmp( psz_cmd, "vtrack" )
+             || !strcmp( psz_cmd, "strack" ) )
+    {
+        char *psz_variable;
+        vlc_value_t val_name;
+        int i_error;
+
+        if( !strcmp( psz_cmd, "atrack" ) )
+        {
+            psz_variable = "audio-es";
+        }
+        else if( !strcmp( psz_cmd, "vtrack" ) )
+        {
+            psz_variable = "video-es";
+        }
+        else
+        {
+            psz_variable = "spu-es";
+        }
+
+        /* Get the descriptive name of the variable */
+        var_Change( p_input, psz_variable, VLC_VAR_GETTEXT,
+                     &val_name, NULL );
+        if( !val_name.psz_string ) val_name.psz_string = strdup(psz_variable);
+
+        if( newval.psz_string && *newval.psz_string )
+        {
+            /* set */
+            vlc_value_t val;
+            val.i_int = atoi( newval.psz_string );
+
+            i_error = var_Set( p_input, psz_variable, val );
+        }
+        else
+        {
+            /* get */
+            vlc_value_t val, text;
+            int i, i_value;
+
+            if ( var_Get( p_input, psz_variable, &val ) < 0 )
+            {
+                vlc_object_release( p_input );
+                return VLC_EGENERIC;
+            }
+            i_value = val.i_int;
+
+            if ( var_Change( p_input, psz_variable,
+                             VLC_VAR_GETLIST, &val, &text ) < 0 )
+            {
+                vlc_object_release( p_input );
+                return VLC_EGENERIC;
+            }
+
+            msg_rc( "+----[ %s ]", val_name.psz_string );
+            for ( i = 0; i < val.p_list->i_count; i++ )
+            {
+                if ( i_value == val.p_list->p_values[i].i_int )
+                    msg_rc( "| %i - %s *", val.p_list->p_values[i].i_int,
+                            text.p_list->p_values[i].psz_string );
+                else
+                    msg_rc( "| %i - %s", val.p_list->p_values[i].i_int,
+                            text.p_list->p_values[i].psz_string );
+            }
+            var_Change( p_input, psz_variable, VLC_VAR_FREELIST,
+                        &val, &text );
+            msg_rc( "+----[ end of %s ]", val_name.psz_string );
+
+            if( val_name.psz_string ) free( val_name.psz_string );
+
+            i_error = VLC_SUCCESS;
+        }
+        vlc_object_release( p_input );
+        return i_error;
+    }
 
     /* Never reached. */
     vlc_object_release( p_input );
@@ -1225,7 +1342,7 @@ static int Playlist( vlc_object_t *p_this, char const *psz_cmd,
         vlc_value_t val;
         var_Get( p_playlist->p_input, "state", &val );
         if( ( val.i_int == PAUSE_S ) || ( val.i_int == PLAYLIST_PAUSED ) )        {
-            msg_rc( _("press menu select or pause to continue") );
+            msg_rc( _("Type 'menu select' or 'pause' to continue.") );
             vlc_object_release( p_playlist );
             return VLC_EGENERIC;
         }
@@ -1245,7 +1362,7 @@ static int Playlist( vlc_object_t *p_this, char const *psz_cmd,
         if( p_playlist->p_input )
         {
             vlc_value_t val;
- 
+
             var_Get( p_playlist->p_input, "rate", &val );
             if( val.i_int != INPUT_RATE_DEFAULT )
             {
@@ -1260,10 +1377,10 @@ static int Playlist( vlc_object_t *p_this, char const *psz_cmd,
     }
     else if (!strcmp( psz_cmd, "goto" ) )
     {
-        if( strlen( newval.psz_string ) > 0) 
+        if( strlen( newval.psz_string ) > 0)
         {
             val.i_int = atoi( newval.psz_string );
-            playlist_Goto( p_playlist, val.i_int); 
+            playlist_Goto( p_playlist, val.i_int);
         }
     }
     else if( !strcmp( psz_cmd, "stop" ) )
@@ -1284,11 +1401,23 @@ static int Playlist( vlc_object_t *p_this, char const *psz_cmd,
 
         if( p_item )
         {
-            msg_rc( "trying to add %s to playlist", newval.psz_string );
+            msg_rc( "Trying to add %s to playlist.", newval.psz_string );
             playlist_AddItem( p_playlist, p_item,
                               PLAYLIST_GO|PLAYLIST_APPEND, PLAYLIST_END );
         }
     }
+    else if( !strcmp( psz_cmd, "enqueue" ) &&
+             newval.psz_string && *newval.psz_string )
+    {
+        playlist_item_t *p_item = parse_MRL( p_intf, newval.psz_string );
+
+        if( p_item )
+        {
+            msg_rc( "trying to enqueue %s to playlist", newval.psz_string );
+            playlist_AddItem( p_playlist, p_item,
+                              PLAYLIST_APPEND, PLAYLIST_END );
+        }
+    }
     else if( !strcmp( psz_cmd, "playlist" ) )
     {
         int i;
@@ -1311,7 +1440,7 @@ static int Playlist( vlc_object_t *p_this, char const *psz_cmd,
         if( p_playlist->p_input )
         {
             /* Replay the current state of the system. */
-            msg_rc( STATUS_CHANGE "( New input: %s )", p_playlist->p_input->input.p_item->psz_uri );
+            msg_rc( STATUS_CHANGE "( new input: %s )", p_playlist->p_input->input.p_item->psz_uri );
             msg_rc( STATUS_CHANGE "( audio volume: %d )", config_GetInt( p_intf, "volume" ));
 
             vlc_mutex_lock( &p_playlist->object_lock );
@@ -1372,7 +1501,7 @@ static int Other( vlc_object_t *p_this, char const *psz_cmd,
         var_Get( p_input, "state", &val );
         if( ( val.i_int == PAUSE_S ) || ( val.i_int == PLAYLIST_PAUSED ) )
         {
-            msg_rc( _("press pause to continue") );
+            msg_rc( _("Type 'pause' to continue.") );
             vlc_object_release( p_playlist );
             vlc_object_release( p_input );
             return VLC_EGENERIC;
@@ -1387,7 +1516,7 @@ static int Other( vlc_object_t *p_this, char const *psz_cmd,
             val.psz_string = newval.psz_string;
             var_Set( p_input->p_libvlc, "marq-marquee", val );
         }
-        else 
+        else
         {
                 val.psz_string = "";
                 var_Set( p_input->p_libvlc, "marq-marquee", val);
@@ -1395,7 +1524,7 @@ static int Other( vlc_object_t *p_this, char const *psz_cmd,
     }
     else if( !strcmp( psz_cmd, "marq-x" ) )
     {
-        if( strlen( newval.psz_string ) > 0) 
+        if( strlen( newval.psz_string ) > 0)
         {
             val.i_int = atoi( newval.psz_string );
             var_Set( p_input->p_libvlc, "marq-x", val );
@@ -1403,7 +1532,7 @@ static int Other( vlc_object_t *p_this, char const *psz_cmd,
     }
     else if( !strcmp( psz_cmd, "marq-y" ) )
     {
-        if( strlen( newval.psz_string ) > 0) 
+        if( strlen( newval.psz_string ) > 0)
         {
             val.i_int = atoi( newval.psz_string );
             var_Set( p_input->p_libvlc, "marq-y", val );
@@ -1411,7 +1540,7 @@ static int Other( vlc_object_t *p_this, char const *psz_cmd,
     }
     else if( !strcmp( psz_cmd, "marq-position" ) )
     {
-        if( strlen( newval.psz_string ) > 0) 
+        if( strlen( newval.psz_string ) > 0)
         {
             val.i_int = atoi( newval.psz_string );
             var_Set( p_input->p_libvlc, "marq-position", val );
@@ -1419,7 +1548,7 @@ static int Other( vlc_object_t *p_this, char const *psz_cmd,
     }
     else if( !strcmp( psz_cmd, "marq-color" ) )
     {
-        if( strlen( newval.psz_string ) > 0) 
+        if( strlen( newval.psz_string ) > 0)
         {
             val.i_int = strtol( newval.psz_string, NULL, 0 );
             var_Set( p_input->p_libvlc, "marq-color", val );
@@ -1427,7 +1556,7 @@ static int Other( vlc_object_t *p_this, char const *psz_cmd,
     }
     else if( !strcmp( psz_cmd, "marq-opacity" ) )
     {
-        if( strlen( newval.psz_string ) > 0) 
+        if( strlen( newval.psz_string ) > 0)
         {
             val.i_int = strtol( newval.psz_string, NULL, 0 );
             var_Set( p_input->p_libvlc, "marq-opacity", val );
@@ -1435,7 +1564,7 @@ static int Other( vlc_object_t *p_this, char const *psz_cmd,
     }
     else if( !strcmp( psz_cmd, "marq-size" ) )
     {
-        if( strlen( newval.psz_string ) > 0) 
+        if( strlen( newval.psz_string ) > 0)
         {
             val.i_int = atoi( newval.psz_string );
             var_Set( p_input->p_libvlc, "marq-size", val );
@@ -1443,7 +1572,7 @@ static int Other( vlc_object_t *p_this, char const *psz_cmd,
     }
     else if( !strcmp( psz_cmd, "marq-timeout" ) )
     {
-        if( strlen( newval.psz_string ) > 0) 
+        if( strlen( newval.psz_string ) > 0)
         {
             val.i_int = atoi( newval.psz_string );
             var_Set( p_input, "marq-timeout", val );
@@ -1552,7 +1681,7 @@ static int Other( vlc_object_t *p_this, char const *psz_cmd,
             val.psz_string = newval.psz_string;
             var_Set( p_input->p_libvlc, "time-format", val );
         }
-        else 
+        else
         {
             val.psz_string = "";
             var_Set( p_input->p_libvlc, "time-format", val);
@@ -1560,7 +1689,7 @@ static int Other( vlc_object_t *p_this, char const *psz_cmd,
     }
     else if( !strcmp( psz_cmd, "time-x" ) )
     {
-        if( strlen( newval.psz_string ) > 0) 
+        if( strlen( newval.psz_string ) > 0)
         {
             val.i_int = atoi( newval.psz_string );
             var_Set( p_input->p_libvlc, "time-x", val );
@@ -1568,7 +1697,7 @@ static int Other( vlc_object_t *p_this, char const *psz_cmd,
     }
     else if( !strcmp( psz_cmd, "time-y" ) )
     {
-        if( strlen( newval.psz_string ) > 0) 
+        if( strlen( newval.psz_string ) > 0)
         {
             val.i_int = atoi( newval.psz_string );
             var_Set( p_input->p_libvlc, "time-y", val );
@@ -1576,7 +1705,7 @@ static int Other( vlc_object_t *p_this, char const *psz_cmd,
     }
     else if( !strcmp( psz_cmd, "time-position" ) )
     {
-        if( strlen( newval.psz_string ) > 0) 
+        if( strlen( newval.psz_string ) > 0)
         {
             val.i_int = atoi( newval.psz_string );
             var_Set( p_input->p_libvlc, "time-position", val );
@@ -1584,7 +1713,7 @@ static int Other( vlc_object_t *p_this, char const *psz_cmd,
     }
     else if( !strcmp( psz_cmd, "time-color" ) )
     {
-        if( strlen( newval.psz_string ) > 0) 
+        if( strlen( newval.psz_string ) > 0)
         {
             val.i_int = strtol( newval.psz_string, NULL, 0 );
             var_Set( p_input->p_libvlc, "time-color", val );
@@ -1592,7 +1721,7 @@ static int Other( vlc_object_t *p_this, char const *psz_cmd,
     }
     else if( !strcmp( psz_cmd, "time-opacity" ) )
     {
-        if( strlen( newval.psz_string ) > 0) 
+        if( strlen( newval.psz_string ) > 0)
         {
             val.i_int = strtol( newval.psz_string, NULL, 0 );
             var_Set( p_input->p_libvlc, "time-opacity", val );
@@ -1600,7 +1729,7 @@ static int Other( vlc_object_t *p_this, char const *psz_cmd,
     }
     else if( !strcmp( psz_cmd, "time-size" ) )
     {
-        if( strlen( newval.psz_string ) > 0) 
+        if( strlen( newval.psz_string ) > 0)
         {
             val.i_int = atoi( newval.psz_string );
             var_Set( p_input->p_libvlc, "time-size", val );
@@ -1616,7 +1745,7 @@ static int Other( vlc_object_t *p_this, char const *psz_cmd,
     }
     else if( !strcmp( psz_cmd, "logo-x" ) )
     {
-        if( strlen( newval.psz_string ) > 0) 
+        if( strlen( newval.psz_string ) > 0)
         {
             val.i_int = atoi( newval.psz_string );
             var_Set( p_input->p_libvlc, "logo-x", val );
@@ -1624,7 +1753,7 @@ static int Other( vlc_object_t *p_this, char const *psz_cmd,
     }
     else if( !strcmp( psz_cmd, "logo-y" ) )
     {
-        if( strlen( newval.psz_string ) > 0) 
+        if( strlen( newval.psz_string ) > 0)
         {
             val.i_int = atoi( newval.psz_string );
             var_Set( p_input->p_libvlc, "logo-y", val );
@@ -1632,7 +1761,7 @@ static int Other( vlc_object_t *p_this, char const *psz_cmd,
     }
     else if( !strcmp( psz_cmd, "logo-position" ) )
     {
-        if( strlen( newval.psz_string ) > 0) 
+        if( strlen( newval.psz_string ) > 0)
         {
             val.i_int = atoi( newval.psz_string );
             var_Set( p_input->p_libvlc, "logo-position", val );
@@ -1640,7 +1769,7 @@ static int Other( vlc_object_t *p_this, char const *psz_cmd,
     }
     else if( !strcmp( psz_cmd, "logo-transparency" ) )
     {
-        if( strlen( newval.psz_string ) > 0) 
+        if( strlen( newval.psz_string ) > 0)
         {
             val.i_int = strtol( newval.psz_string, NULL, 0 );
             var_Set( p_input->p_libvlc, "logo-transparency", val );
@@ -1652,7 +1781,7 @@ static int Other( vlc_object_t *p_this, char const *psz_cmd,
      */
     else
     {
-        msg_rc( "unknown command!" );
+        msg_rc( "Unknown command!" );
     }
 
     vlc_object_release( p_playlist );
@@ -1670,7 +1799,7 @@ static int Quit( vlc_object_t *p_this, char const *psz_cmd,
     {
         playlist_Stop( p_playlist );
         vlc_object_release( p_playlist );
-    }    
+    }
     p_this->p_vlc->b_die = VLC_TRUE;
     return VLC_SUCCESS;
 }
@@ -1680,7 +1809,7 @@ static int Intf( vlc_object_t *p_this, char const *psz_cmd,
 {
     intf_thread_t *p_newintf = NULL;
 
-    p_newintf = intf_Create( p_this->p_vlc, newval.psz_string );
+    p_newintf = intf_Create( p_this->p_vlc, newval.psz_string, 0, NULL );
     if( p_newintf )
     {
         p_newintf->b_block = VLC_FALSE;
@@ -1712,7 +1841,7 @@ static int Volume( vlc_object_t *p_this, char const *psz_cmd,
         var_Get( p_input, "state", &val );
         if( ( val.i_int == PAUSE_S ) || ( val.i_int == PLAYLIST_PAUSED ) )
         {
-            msg_rc( _("press menu select or pause to continue") );
+            msg_rc( _("Type 'menu select' or 'pause' to continue.") );
             vlc_object_release( p_input );
             return VLC_EGENERIC;
         }
@@ -1725,7 +1854,7 @@ static int Volume( vlc_object_t *p_this, char const *psz_cmd,
         audio_volume_t i_volume = atoi( newval.psz_string );
         if ( (i_volume > (audio_volume_t)AOUT_VOLUME_MAX) )
         {
-            msg_rc( "Volume must be in the range %d-%d", AOUT_VOLUME_MIN,
+            msg_rc( "Volume must be in the range %d-%d.", AOUT_VOLUME_MIN,
                     AOUT_VOLUME_MAX );
             i_error = VLC_EBADVAR;
         }
@@ -1782,7 +1911,7 @@ static int VolumeMove( vlc_object_t *p_this, char const *psz_cmd,
         var_Get( p_input, "state", &val );
         if( ( val.i_int == PAUSE_S ) || ( val.i_int == PLAYLIST_PAUSED ) )
         {
-            msg_rc( _("press menu select or pause to continue") );
+            msg_rc( _("Type 'menu select' or 'pause' to continue.") );
             vlc_object_release( p_input );
             return VLC_EGENERIC;
         }
@@ -1811,6 +1940,126 @@ static int VolumeMove( vlc_object_t *p_this, char const *psz_cmd,
     return i_error;
 }
 
+
+static int VideoConfig( vlc_object_t *p_this, char const *psz_cmd,
+                        vlc_value_t oldval, vlc_value_t newval, void *p_data )
+{
+    intf_thread_t *p_intf = (intf_thread_t*)p_this;
+    input_thread_t *p_input = NULL;
+    vout_thread_t * p_vout;
+    const char * psz_variable;
+    vlc_value_t val_name;
+    int i_error;
+
+    p_input = vlc_object_find( p_this, VLC_OBJECT_INPUT, FIND_ANYWHERE );
+    if( !p_input )
+        return VLC_ENOOBJ;
+
+    p_vout = vlc_object_find( p_input, VLC_OBJECT_VOUT, FIND_CHILD );
+    vlc_object_release( p_input );
+    if( !p_vout )
+        return VLC_ENOOBJ;
+
+    if( !strcmp( psz_cmd, "vcrop" ) )
+    {
+        psz_variable = "crop";
+    }
+    else if( !strcmp( psz_cmd, "vratio" ) )
+    {
+        psz_variable = "aspect-ratio";
+    }
+    else /* if( !strcmp( psz_cmd, "vzoom" ) ) */
+    {
+        psz_variable = "zoom";
+    }
+
+
+    /* Get the descriptive name of the variable */
+    var_Change( p_vout, psz_variable, VLC_VAR_GETTEXT,
+                 &val_name, NULL );
+    if( !val_name.psz_string ) val_name.psz_string = strdup(psz_variable);
+
+    if( newval.psz_string && *newval.psz_string )
+    {
+        /* set */
+        if( !strcmp( psz_variable, "zoom" ) )
+        {
+            vlc_value_t val;
+            val.f_float = atof( newval.psz_string );
+            i_error = var_Set( p_vout, psz_variable, val );
+        }
+        else
+        {
+            i_error = var_Set( p_vout, psz_variable, newval );
+        }
+    }
+    else
+    {
+        /* get */
+        vlc_value_t val, text;
+        int i;
+        float f_value = 0.0;
+        char *psz_value = NULL;
+
+        if ( var_Get( p_vout, psz_variable, &val ) < 0 )
+        {
+            vlc_object_release( p_vout );
+            return VLC_EGENERIC;
+        }
+        if( !strcmp( psz_variable, "zoom" ) )
+        {
+            f_value = val.f_float;
+        }
+        else
+        {
+            psz_value = strdup( val.psz_string );
+        }
+
+        if ( var_Change( p_vout, psz_variable,
+                         VLC_VAR_GETLIST, &val, &text ) < 0 )
+        {
+            vlc_object_release( p_vout );
+            return VLC_EGENERIC;
+        }
+
+        msg_rc( "+----[ %s ]", val_name.psz_string );
+        if( !strcmp( psz_variable, "zoom" ) )
+        {
+            for ( i = 0; i < val.p_list->i_count; i++ )
+            {
+                if ( f_value == val.p_list->p_values[i].f_float )
+                    msg_rc( "| %f - %s *", val.p_list->p_values[i].f_float,
+                            text.p_list->p_values[i].psz_string );
+                else
+                    msg_rc( "| %f - %s", val.p_list->p_values[i].f_float,
+                            text.p_list->p_values[i].psz_string );
+            }
+        }
+        else
+        {
+            for ( i = 0; i < val.p_list->i_count; i++ )
+            {
+                if ( !strcmp( psz_value, val.p_list->p_values[i].psz_string ) )
+                    msg_rc( "| %s - %s *", val.p_list->p_values[i].psz_string,
+                            text.p_list->p_values[i].psz_string );
+                else
+                    msg_rc( "| %s - %s", val.p_list->p_values[i].psz_string,
+                            text.p_list->p_values[i].psz_string );
+            }
+            free( psz_value );
+        }
+        var_Change( p_vout, psz_variable, VLC_VAR_FREELIST,
+                    &val, &text );
+        msg_rc( "+----[ end of %s ]", val_name.psz_string );
+
+        if( val_name.psz_string ) free( val_name.psz_string );
+
+        i_error = VLC_SUCCESS;
+    }
+    vlc_object_release( p_vout );
+    return i_error;
+}
+
 static int AudioConfig( vlc_object_t *p_this, char const *psz_cmd,
                         vlc_value_t oldval, vlc_value_t newval, void *p_data )
 {
@@ -1831,7 +2080,7 @@ static int AudioConfig( vlc_object_t *p_this, char const *psz_cmd,
 
         var_Get( p_input, "state", &val );
         if( ( val.i_int == PAUSE_S ) || ( val.i_int == PLAYLIST_PAUSED ) )        {
-            msg_rc( _("press menu select or pause to continue") );
+            msg_rc( _("Type 'menu select' or 'pause' to continue.") );
             vlc_object_release( p_input );
             return VLC_EGENERIC;
         }
@@ -1915,7 +2164,7 @@ static int Menu( vlc_object_t *p_this, char const *psz_cmd,
 
     if ( !*newval.psz_string )
     {
-        msg_rc( _("please provide one of the following paramaters") );
+        msg_rc( _("Please provide one of the following parameters:") );
         msg_rc( "[on|off|up|down|left|right|select]" );
         return i_error;
     }
@@ -1930,7 +2179,7 @@ static int Menu( vlc_object_t *p_this, char const *psz_cmd,
         if( ( ( val.i_int == PAUSE_S ) || ( val.i_int == PLAYLIST_PAUSED ) ) &&
             ( strcmp( newval.psz_string, "select" ) != 0 ) )
         {
-            msg_rc( _("press menu select or pause to continue") );
+            msg_rc( _("Type 'menu select' or 'pause' to continue.") );
             vlc_object_release( p_playlist );
             return VLC_EGENERIC;
         }
@@ -1954,7 +2203,7 @@ static int Menu( vlc_object_t *p_this, char const *psz_cmd,
         osd_MenuActivate( p_this );
     else
     {
-        msg_rc( _("please provide one of the following paramaters") );
+        msg_rc( _("Please provide one of the following parameters:") );
         msg_rc( "[on|off|up|down|left|right|select]" );
         if( val.psz_string ) free( val.psz_string );
             return i_error;
@@ -2150,3 +2399,120 @@ static playlist_item_t *parse_MRL( intf_thread_t *p_intf, char *psz_mrl )
 
     return p_item;
 }
+
+/*****************************************************************************
+ * checkUpdates : check for updates
+ ****************************************************************************/
+static void checkUpdates( intf_thread_t *p_intf, char *psz_arg )
+{
+    update_iterator_t *p_uit;
+    update_t *p_u = update_New( p_intf );
+    if( p_u == NULL ) return;
+    p_uit = update_iterator_New( p_u );
+    if( p_uit )
+    {
+        int s = 0, t = 0;
+
+        if( strstr( psz_arg, "newer" ) )
+            s |= UPDATE_RELEASE_STATUS_NEWER;
+        if( strstr( psz_arg, "equal" ) )
+            s |= UPDATE_RELEASE_STATUS_EQUAL;
+        if( strstr( psz_arg, "older" ) )
+            s |= UPDATE_RELEASE_STATUS_OLDER;
+        if( s ) p_uit->i_rs = s;
+        else p_uit->i_rs = UPDATE_RELEASE_STATUS_NEWER;
+
+        if( strstr( psz_arg, "undef" ) )
+            t |= UPDATE_FILE_TYPE_UNDEF;
+        if( strstr( psz_arg, "info" ) )
+            t |= UPDATE_FILE_TYPE_INFO;
+        if( strstr( psz_arg, "source" ) )
+            t |= UPDATE_FILE_TYPE_SOURCE;
+        if( strstr( psz_arg, "binary" ) )
+            t |= UPDATE_FILE_TYPE_BINARY;
+        if( strstr( psz_arg, "plugin" ) )
+            t |= UPDATE_FILE_TYPE_PLUGIN;
+        if( t ) p_uit->i_t = t;
+
+        update_Check( p_u, VLC_FALSE );
+        update_iterator_Action( p_uit, UPDATE_MIRROR );
+        msg_rc( "\nUsing mirror: %s (%s) [%s]",
+                p_uit->mirror.psz_name,
+                p_uit->mirror.psz_location,
+                p_uit->mirror.psz_type );
+        while( (s = update_iterator_Action( p_uit, UPDATE_FILE )) != UPDATE_FAIL )
+        {
+            char *psz_tmp;
+            if( s & UPDATE_RELEASE )
+            {
+                switch( p_uit->release.i_status )
+                {
+                    case UPDATE_RELEASE_STATUS_OLDER:
+                        psz_tmp = strdup( "older" );
+                        break;
+                    case UPDATE_RELEASE_STATUS_EQUAL:
+                        psz_tmp = strdup( "equal" );
+                        break;
+                    case UPDATE_RELEASE_STATUS_NEWER:
+                        psz_tmp = strdup( "newer" );
+                        break;
+                    default:
+                        psz_tmp = strdup( "?!?" );
+                        break;
+                }
+                msg_rc( "\n+----[ VLC %s %s (%s) ] ",
+                        p_uit->release.psz_version,
+                        p_uit->release.psz_svn_revision,
+                        psz_tmp );
+                free( psz_tmp );
+            }
+            switch( p_uit->file.i_type )
+            {
+                case UPDATE_FILE_TYPE_UNDEF:
+                    psz_tmp = strdup( "undef" );
+                    break;
+                case UPDATE_FILE_TYPE_INFO:
+                    psz_tmp = strdup( "info" );
+                    break;
+                case UPDATE_FILE_TYPE_SOURCE:
+                    psz_tmp = strdup( "source" );
+                    break;
+                case UPDATE_FILE_TYPE_BINARY:
+                    psz_tmp = strdup( "binary" );
+                    break;
+                case UPDATE_FILE_TYPE_PLUGIN:
+                    psz_tmp = strdup( "plugin" );
+                    break;
+                default:
+                    psz_tmp = strdup( "?!?" );
+                    break;
+            }
+            msg_rc( "| %s (%s)", p_uit->file.psz_description, psz_tmp );
+            free( psz_tmp );
+            if( p_uit->file.l_size )
+            {
+                if( p_uit->file.l_size > 1024 * 1024 * 1024 )
+                    asprintf( &psz_tmp, "(%ld GB)",
+                              p_uit->file.l_size / (1024*1024*1024) );
+                if( p_uit->file.l_size > 1024 * 1024 )
+                    asprintf( &psz_tmp, "(%ld MB)",
+                              p_uit->file.l_size / (1024*1024) );
+                else if( p_uit->file.l_size > 1024 )
+                    asprintf( &psz_tmp, "(%ld kB)",
+                              p_uit->file.l_size / 1024 );
+                else
+                    asprintf( &psz_tmp, "(%ld B)", p_uit->file.l_size );
+            }
+            else
+            {
+                psz_tmp = strdup( "" );
+            }
+            msg_rc( "| %s %s", p_uit->file.psz_url, psz_tmp );
+            msg_rc( "+----" );
+            free( psz_tmp );
+        }
+        msg_rc( "" );
+        update_iterator_Delete( p_uit );
+    }
+    update_Delete( p_u );
+}
