@@ -2,7 +2,7 @@
  * telnet.c: VLM interface plugin
  *****************************************************************************
  * Copyright (C) 2000-2006 the VideoLAN team
- * $Id: telnet.c 15002 2006-03-31 16:12:31Z fkuehne $
+ * $Id: telnet.c 19602 2007-04-01 01:10:23Z hartman $
  *
  * Authors: Simon Latapie <garf@videolan.org>
  *          Laurent Aimar <fenrir@videolan.org>
@@ -137,7 +137,7 @@ struct intf_sys_t
  * option is missing from --telnet-host.
  */
 static int getPort(intf_thread_t *p_intf, vlc_url_t url, int i_port)
-    {
+{
     // Print error if two different ports have been specified
     if (url.i_port != 0  &&
         i_port != TELNETPORT_DEFAULT && 
@@ -154,7 +154,6 @@ static int getPort(intf_thread_t *p_intf, vlc_url_t url, int i_port)
     {
  	    return url.i_port;
     }
-    // return default
     return i_port;
 }
 
@@ -383,7 +382,7 @@ static void Run( intf_thread_t *p_intf )
                                    cl->i_mode + 2 );
                 }
 
-                if (i_recv == 0)
+                if( i_recv == 0  || ( i_recv == -1 &&  errno != EAGAIN && errno != 0 ) )
                 {
                     net_Close( cl->fd );
                     TAB_REMOVE( p_intf->p_sys->i_clients ,
