@@ -2,7 +2,7 @@
  * ps.h: Program Stream demuxer helper
  *****************************************************************************
  * Copyright (C) 2004 the VideoLAN team
- * $Id: ps.h 13905 2006-01-12 23:10:04Z dionoea $
+ * $Id: ps.h 17183 2006-10-20 21:33:16Z hartman $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -36,6 +36,8 @@ typedef struct
     int         i_id;
     es_out_id_t *es;
     es_format_t fmt;
+    mtime_t     i_first_pts;
+    mtime_t     i_last_pts;
 
 } ps_track_t;
 
@@ -49,6 +51,8 @@ static inline void ps_track_init( ps_track_t tk[PS_TK_COUNT] )
         tk[i].i_skip = 0;
         tk[i].i_id   = 0;
         tk[i].es     = NULL;
+        tk[i].i_first_pts = -1;
+        tk[i].i_last_pts = -1;
         es_format_Init( &tk[i].fmt, UNKNOWN_ES, 0 );
     }
 }
