@@ -2,11 +2,12 @@
  * intf.h: MacOS X interface module
  *****************************************************************************
  * Copyright (C) 2002-2006 the VideoLAN team
- * $Id: intf.h 15247 2006-04-16 16:06:16Z fkuehne $
+ * $Id: intf.h 17496 2006-11-05 21:53:18Z fkuehne $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Christophe Massiot <massiot@via.ecp.fr>
  *          Derk-Jan Hartman <hartman at videolan dot org>
+ *          Felix Khne <fkuehne at videolan dot org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -86,6 +87,7 @@ struct intf_sys_t
 /*****************************************************************************
  * VLCMain interface
  *****************************************************************************/
+@class AppleRemote;
 @interface VLCMain : NSObject
 {
     intf_thread_t *p_intf;      /* The main intf object */
@@ -266,6 +268,18 @@ struct intf_sys_t
     IBOutlet id o_dmi_previous;
     IBOutlet id o_dmi_mute;
 
+    /* vout menu */
+    IBOutlet id o_vout_menu;
+    IBOutlet id o_vmi_play;
+    IBOutlet id o_vmi_stop;
+    IBOutlet id o_vmi_prev;
+    IBOutlet id o_vmi_next;
+    IBOutlet id o_vmi_volup;
+    IBOutlet id o_vmi_voldown;
+    IBOutlet id o_vmi_mute;
+    IBOutlet id o_vmi_fullscreen;
+    IBOutlet id o_vmi_snapshot;
+
     bool b_small_window;
 
     mtime_t i_end_scroll;
@@ -273,6 +287,9 @@ struct intf_sys_t
     NSSize o_size_with_playlist;
 
     int     i_lastShownVolume;
+
+    AppleRemote * o_remote;
+    BOOL b_left_right_remote_button_hold; /* true as long as the user holds the left or right button on the remote control */
 }
 
 + (VLCMain *)sharedInstance;
@@ -287,6 +304,7 @@ struct intf_sys_t
 - (id)getBookmarks;
 - (id)getEmbeddedList;
 - (id)getInteractionList;
+- (id)getVoutMenu;
 - (void)terminate;
 - (NSString *)localizedString:(char *)psz;
 - (char *)delocalizeString:(NSString *)psz;
