@@ -1,11 +1,11 @@
 /*****************************************************************************
  * evt_special.hpp
  *****************************************************************************
- * Copyright (C) 2003 VideoLAN
- * $Id: evt_special.hpp 9641 2004-12-22 13:06:43Z gbazin $
+ * Copyright (C) 2003 the VideoLAN team
+ * $Id: evt_special.hpp 14187 2006-02-07 16:37:40Z courmisch $
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
- *          Olivier Teulire <ipkiss@via.ecp.fr>
+ *          Olivier Teulière <ipkiss@via.ecp.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 #ifndef EVT_SPECIAL_HPP
@@ -32,13 +32,13 @@
 class EvtSpecial: public EvtGeneric
 {
     public:
-        typedef enum
+        enum ActionType_t
         {
             kShow,
             kHide,
             kEnable,
             kDisable
-        } ActionType_t;
+        };
 
         EvtSpecial( intf_thread_t *pIntf, ActionType_t action ):
             EvtGeneric( pIntf ), m_action( action ) {}
