@@ -1,10 +1,10 @@
 /*****************************************************************************
- * sap.c :  SAP interface module
+ * hal.c :  HAL interface module
  *****************************************************************************
  * Copyright (C) 2004 the VideoLAN team
- * $Id: sap.c 9217 2004-11-07 11:02:59Z courmisch $
+ * $Id: hal.c 17886 2006-11-20 04:01:08Z funman $
  *
- * Authors: Clment Stenac <zorglub@videolan.org>
+ * Authors: Clément Stenac <zorglub@videolan.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -60,7 +60,7 @@
     static void Close( vlc_object_t * );
 
 vlc_module_begin();
-    set_description( _("HAL device detection") );
+    set_description( _("HAL devices detection") );
     set_category( CAT_PLAYLIST );
     set_subcategory( SUBCAT_PLAYLIST_SD );
 
@@ -103,13 +103,39 @@ static int Open( vlc_object_t *p_this )
     playlist_t          *p_playlist;
     playlist_view_t     *p_view;
 
+    DBusError           dbus_error;
+    DBusConnection      *p_connection;
+
     p_sd->pf_run = Run;
     p_sd->p_sys  = p_sys;
 
-    if( !( p_sys->p_ctx = hal_initialize( NULL, FALSE ) ) )
+    dbus_error_init( &dbus_error );
+
+#ifdef HAVE_HAL_1
+    p_sys->p_ctx = libhal_ctx_new();
+    if( !p_sys->p_ctx )
     {
+        msg_Err( p_sd, "unable to create HAL context") ;
+        free( p_sys );
+        return VLC_EGENERIC;
+    }
+    p_connection = dbus_bus_get( DBUS_BUS_SYSTEM, &dbus_error );
+    if( dbus_error_is_set( &dbus_error ) )
+    {
+        msg_Err( p_sd, "unable to connect to DBUS: %s", dbus_error.message );
+        dbus_error_free( &dbus_error );
+        free( p_sys );
+        return VLC_EGENERIC;
+    }
+    libhal_ctx_set_dbus_connection( p_sys->p_ctx, p_connection );
+    if( !libhal_ctx_init( p_sys->p_ctx, &dbus_error ) )
+#else
+    if( !(p_sys->p_ctx = hal_initialize( NULL, FALSE ) ) )
+#endif
+    {
+        msg_Err( p_sd, "hal not available : %s", dbus_error.message );
+        dbus_error_free( &dbus_error );
         free( p_sys );
-        msg_Err( p_sd, "hal not available" );
         return VLC_EGENERIC;
     }
 
@@ -160,16 +186,27 @@ static void AddDvd( services_discovery_t *p_sd, char *psz_device )
     services_discovery_sys_t    *p_sys  = p_sd->p_sys;
     playlist_t          *p_playlist;
     playlist_item_t     *p_item;
+#ifdef HAVE_HAL_1
+    psz_name = libhal_device_get_property_string( p_sd->p_sys->p_ctx,
+                                        psz_device, "volume.label", NULL );
+    psz_blockdevice = libhal_device_get_property_string( p_sd->p_sys->p_ctx,
+                                        psz_device, "block.device", NULL );
+#else
     psz_name = hal_device_get_property_string( p_sd->p_sys->p_ctx,
                                                psz_device, "volume.label" );
     psz_blockdevice = hal_device_get_property_string( p_sd->p_sys->p_ctx,
                                                  psz_device, "block.device" );
+#endif
     asprintf( &psz_uri, "dvd://%s", psz_blockdevice );
     /* Create the playlist item here */
     p_item = playlist_ItemNew( p_sd, psz_uri,
                                psz_name );
     free( psz_uri );
+#ifdef HAVE_HAL_1
+    libhal_free_string( psz_device );
+#else
     hal_free_string( psz_device );
+#endif
     if( !p_item )
     {
         return;
@@ -197,14 +234,23 @@ static void AddCdda( services_discovery_t *p_sd, char *psz_device )
     services_discovery_sys_t    *p_sys  = p_sd->p_sys;
     playlist_t          *p_playlist;
     playlist_item_t     *p_item;
+#ifdef HAVE_HAL_1
+    psz_blockdevice = libhal_device_get_property_string( p_sd->p_sys->p_ctx,
+                                            psz_device, "block.device", NULL );
+#else
     psz_blockdevice = hal_device_get_property_string( p_sd->p_sys->p_ctx,
                                                  psz_device, "block.device" );
+#endif
     asprintf( &psz_uri, "cdda://%s", psz_blockdevice );
     /* Create the playlist item here */
     p_item = playlist_ItemNew( p_sd, psz_uri,
                                psz_name );
     free( psz_uri );
+#ifdef HAVE_HAL_1
+    libhal_free_string( psz_device );
+#else
     hal_free_string( psz_device );
+#endif
     if( !p_item )
     {
         return;
@@ -229,24 +275,48 @@ static void ParseDevice( services_discovery_t *p_sd, char *psz_device )
 {
     char *psz_disc_type;
     services_discovery_sys_t    *p_sys  = p_sd->p_sys;
+#ifdef HAVE_HAL_1
+    if( libhal_device_property_exists( p_sys->p_ctx, psz_device,
+                                       "volume.disc.type", NULL ) )
+    {
+        psz_disc_type = libhal_device_get_property_string( p_sys->p_ctx,
+                                                        psz_device,
+                                                        "volume.disc.type",
+                                                        NULL );
+#else
     if( hal_device_property_exists( p_sys->p_ctx, psz_device,
                                     "volume.disc.type" ) )
     {
         psz_disc_type = hal_device_get_property_string( p_sys->p_ctx,
                                                         psz_device,
                                                         "volume.disc.type" );
-        if( !strcmp( psz_disc_type, "dvd_rom" ) )
+#endif
+        if( !strncmp( psz_disc_type, "dvd_r", 5 ) )
         {
-            AddDvd( p_sd, psz_device );
+#ifdef HAVE_HAL_1
+            if (libhal_device_get_property_bool( p_sys->p_ctx, psz_device,
+                        "volume.disc.is_videodvd", NULL ) )
+#endif
+                AddDvd( p_sd, psz_device );
         }
-        else if( !strcmp( psz_disc_type, "cd_rom" ) )
+        else if( !strncmp( psz_disc_type, "cd_r", 4 ) )
         {
-            if( hal_device_get_property_bool( p_sys->p_ctx, psz_device, "volume.disc.has_audio" ) )
+#ifdef HAVE_HAL_1
+            if( libhal_device_get_property_bool( p_sys->p_ctx, psz_device,
+                                         "volume.disc.has_audio" , NULL ) )
+#else
+            if( hal_device_get_property_bool( p_sys->p_ctx, psz_device,
+                                         "volume.disc.has_audio" ) )
+#endif
             {
                 AddCdda( p_sd, psz_device );
             }
         }
+#ifdef HAVE_HAL_1
+        libhal_free_string( psz_disc_type );
+#else
         hal_free_string( psz_disc_type );
+#endif
     }
 }
 
@@ -260,7 +330,11 @@ static void Run( services_discovery_t *p_sd )
     services_discovery_sys_t    *p_sys  = p_sd->p_sys;
 
     /* parse existing devices first */
+#ifdef HAVE_HAL_1
+    if( ( devices = libhal_get_all_devices( p_sys->p_ctx, &i_devices, NULL ) ) )
+#else
     if( ( devices = hal_get_all_devices( p_sys->p_ctx, &i_devices ) ) )
+#endif
     {
         for( i = 0; i < i_devices; i++ )
         {
