@@ -2,7 +2,7 @@
  * marq.c : marquee display video plugin for vlc
  *****************************************************************************
  * Copyright (C) 2003-2005 the VideoLAN team
- * $Id: marq.c 13324 2005-11-22 21:13:42Z gbazin $
+ * $Id: marq.c 17012 2006-10-09 22:11:32Z xtophe $
  *
  * Authors: Mark Moriarty
  *
@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -66,40 +66,41 @@ struct filter_sys_t
 
     char *psz_marquee;    /* marquee string */
 
-    int  i_font_color, i_font_opacity, i_font_size; /* font control */
+    text_style_t *p_style; /* font control */
 
     time_t last_time;
 
     vlc_bool_t b_need_update;
 };
 
-#define MSG_TEXT N_("Marquee text")
-#define MSG_LONGTEXT N_("Marquee text to display")
-#define POSX_TEXT N_("X offset, from left")
-#define POSX_LONGTEXT N_("X offset, from the left screen edge" )
-#define POSY_TEXT N_("Y offset, from the top")
-#define POSY_LONGTEXT N_("Y offset, down from the top" )
-#define TIMEOUT_TEXT N_("Marquee timeout")
-#define TIMEOUT_LONGTEXT N_("Defines the time the marquee must remain " \
-                            "displayed, in milliseconds. Default value is " \
-                            "0 (remain forever).")
+#define MSG_TEXT N_("Text")
+#define MSG_LONGTEXT N_("Marquee text to display.")
+#define POSX_TEXT N_("X offset")
+#define POSX_LONGTEXT N_("X offset, from the left screen edge." )
+#define POSY_TEXT N_("Y offset")
+#define POSY_LONGTEXT N_("Y offset, down from the top." )
+#define TIMEOUT_TEXT N_("Timeout")
+#define TIMEOUT_LONGTEXT N_("Number of milliseconds the marquee must remain " \
+                            "displayed. Default value is " \
+                            "0 (remains forever).")
 #define OPACITY_TEXT N_("Opacity")
-#define OPACITY_LONGTEXT N_("The opacity (inverse of transparency) of " \
-    "overlay text. 0 = transparent, 255 = totally opaque. " )
+#define OPACITY_LONGTEXT N_("Opacity (inverse of transparency) of " \
+    "overlayed text. 0 = transparent, 255 = totally opaque. " )
 #define SIZE_TEXT N_("Font size, pixels")
-#define SIZE_LONGTEXT N_("Specify the font size, in pixels, " \
-    "with -1 = use freetype-fontsize" )
+#define SIZE_LONGTEXT N_("Font size, in pixels. Default is -1 (use default " \
+    "font size)." )
 
-#define COLOR_TEXT N_("Text Default Color")
-#define COLOR_LONGTEXT N_("The color of overlay text. 1 byte for each color, hexadecimal. " \
-    "#000000 = all colors off, " \
-    "0xFF0000 = just Red, 0xFFFFFF = all color on [White]" )
+#define COLOR_TEXT N_("Color")
+#define COLOR_LONGTEXT N_("Color of the text that will be rendered on "\
+    "the video. This must be an hexadecimal (like HTML colors). The first two "\
+    "chars are for red, then green, then blue. #000000 = black, #FF0000 = red,"\
+    " #00FF00 = green, #FFFF00 = yellow (red + green), #FFFFFF = white" )
 
 #define POS_TEXT N_("Marquee position")
 #define POS_LONGTEXT N_( \
   "You can enforce the marquee position on the video " \
   "(0=center, 1=left, 2=right, 4=top, 8=bottom, you can " \
-  "also use combinations of these values by adding them).")
+  "also use combinations of these values, eg 6 = top-right).")
 
 static int pi_pos_values[] = { 0, 1, 2, 4, 8, 5, 6, 9, 10 };
 static char *ppsz_pos_descriptions[] =
@@ -111,7 +112,7 @@ static char *ppsz_pos_descriptions[] =
  *****************************************************************************/
 vlc_module_begin();
     set_capability( "sub filter", 0 );
-    set_shortname( N_("Marquee" ));
+    set_shortname( _("Marquee" ));
     set_callbacks( CreateFilter, DestroyFilter );
     set_category( CAT_VIDEO );
     set_subcategory( SUBCAT_VIDEO_SUBPIC );
@@ -137,7 +138,7 @@ vlc_module_begin();
     add_integer( "marq-timeout", 0, NULL, TIMEOUT_TEXT, TIMEOUT_LONGTEXT,
                  VLC_FALSE );
 
-    set_description( _("Marquee display sub filter") );
+    set_description( _("Marquee display") );
     add_shortcut( "marq" );
 vlc_module_end();
 
@@ -148,7 +149,6 @@ static int CreateFilter( vlc_object_t *p_this )
 {
     filter_t *p_filter = (filter_t *)p_this;
     filter_sys_t *p_sys;
-    vlc_object_t *p_input;
 
     /* Allocate structure */
     p_sys = p_filter->p_sys = malloc( sizeof( filter_sys_t ) );
@@ -158,34 +158,26 @@ static int CreateFilter( vlc_object_t *p_this )
         return VLC_ENOMEM;
     }
 
-    /* Hook used for callback variables */
-    p_input = vlc_object_find( p_this, VLC_OBJECT_INPUT, FIND_ANYWHERE );
-    if( !p_input )
-    {
-        return VLC_ENOOBJ;
-    }
+    p_sys->p_style = malloc( sizeof( text_style_t ) );
+    memcpy( p_sys->p_style, &default_text_style, sizeof( text_style_t ) );
 
-    p_sys->i_xoff = var_CreateGetInteger( p_input->p_libvlc , "marq-x" );
-    p_sys->i_yoff = var_CreateGetInteger( p_input->p_libvlc , "marq-y" );
-    p_sys->i_timeout = var_CreateGetInteger( p_input->p_libvlc , "marq-timeout" );
-    p_sys->i_pos = var_CreateGetInteger( p_input->p_libvlc , "marq-position" );
-    p_sys->psz_marquee =  var_CreateGetString( p_input->p_libvlc, "marq-marquee" );
-    var_Create( p_input->p_libvlc, "marq-opacity", VLC_VAR_INTEGER|VLC_VAR_DOINHERIT );
-    p_sys->i_font_opacity = var_CreateGetInteger( p_input->p_libvlc , "marq-opacity" );
-    p_sys->i_font_color = var_CreateGetInteger( p_input->p_libvlc , "marq-color" );
-    p_sys->i_font_size = var_CreateGetInteger( p_input->p_libvlc , "marq-size" );
-
-    var_AddCallback( p_input->p_libvlc, "marq-x", MarqueeCallback, p_sys );
-    var_AddCallback( p_input->p_libvlc, "marq-y", MarqueeCallback, p_sys );
-    var_AddCallback( p_input->p_libvlc, "marq-marquee", MarqueeCallback, p_sys );
-    var_AddCallback( p_input->p_libvlc, "marq-timeout", MarqueeCallback, p_sys );
-    var_AddCallback( p_input->p_libvlc, "marq-position", MarqueeCallback, p_sys );
-    var_AddCallback( p_input->p_libvlc, "marq-color", MarqueeCallback, p_sys );
-    var_AddCallback( p_input->p_libvlc, "marq-opacity", MarqueeCallback, p_sys );
-    var_AddCallback( p_input->p_libvlc, "marq-size", MarqueeCallback, p_sys );
-
-    vlc_object_release( p_input );
+    p_sys->i_xoff = var_CreateGetInteger( p_filter->p_libvlc , "marq-x" );
+    p_sys->i_yoff = var_CreateGetInteger( p_filter->p_libvlc , "marq-y" );
+    p_sys->i_timeout = var_CreateGetInteger( p_filter->p_libvlc , "marq-timeout" );
+    p_sys->i_pos = var_CreateGetInteger( p_filter->p_libvlc , "marq-position" );
+    p_sys->psz_marquee =  var_CreateGetString( p_filter->p_libvlc, "marq-marquee" );
+    p_sys->p_style->i_font_alpha = 255 - var_CreateGetInteger( p_filter->p_libvlc , "marq-opacity" );
+    p_sys->p_style->i_font_color = var_CreateGetInteger( p_filter->p_libvlc , "marq-color" );
+    p_sys->p_style->i_font_size  = var_CreateGetInteger( p_filter->p_libvlc , "marq-size" );
 
+    var_AddCallback( p_filter->p_libvlc, "marq-x", MarqueeCallback, p_sys );
+    var_AddCallback( p_filter->p_libvlc, "marq-y", MarqueeCallback, p_sys );
+    var_AddCallback( p_filter->p_libvlc, "marq-marquee", MarqueeCallback, p_sys );
+    var_AddCallback( p_filter->p_libvlc, "marq-timeout", MarqueeCallback, p_sys );
+    var_AddCallback( p_filter->p_libvlc, "marq-position", MarqueeCallback, p_sys );
+    var_AddCallback( p_filter->p_libvlc, "marq-color", MarqueeCallback, p_sys );
+    var_AddCallback( p_filter->p_libvlc, "marq-opacity", MarqueeCallback, p_sys );
+    var_AddCallback( p_filter->p_libvlc, "marq-size", MarqueeCallback, p_sys );
 
     /* Misc init */
     p_filter->pf_sub_filter = Filter;
@@ -201,36 +193,29 @@ static void DestroyFilter( vlc_object_t *p_this )
 {
     filter_t *p_filter = (filter_t *)p_this;
     filter_sys_t *p_sys = p_filter->p_sys;
-    vlc_object_t *p_input;
 
+    if( p_sys->p_style ) free( p_sys->p_style );
     if( p_sys->psz_marquee ) free( p_sys->psz_marquee );
     free( p_sys );
 
     /* Delete the marquee variables */
-    p_input = vlc_object_find( p_this, VLC_OBJECT_INPUT, FIND_ANYWHERE );
-    if( !p_input )
-    {
-        return;
-    }
-    var_DelCallback( p_input->p_libvlc, "marq-x", MarqueeCallback, p_sys );
-    var_DelCallback( p_input->p_libvlc, "marq-y", MarqueeCallback, p_sys );
-    var_DelCallback( p_input->p_libvlc, "marq-marquee", MarqueeCallback, p_sys );
-    var_DelCallback( p_input->p_libvlc, "marq-timeout", MarqueeCallback, p_sys );
-    var_DelCallback( p_input->p_libvlc, "marq-position", MarqueeCallback, p_sys );
-    var_DelCallback( p_input->p_libvlc, "marq-color", MarqueeCallback, p_sys );
-    var_DelCallback( p_input->p_libvlc, "marq-opacity", MarqueeCallback, p_sys );
-    var_DelCallback( p_input->p_libvlc, "marq-size", MarqueeCallback, p_sys );
+    var_DelCallback( p_filter->p_libvlc, "marq-x", MarqueeCallback, p_sys );
+    var_DelCallback( p_filter->p_libvlc, "marq-y", MarqueeCallback, p_sys );
+    var_DelCallback( p_filter->p_libvlc, "marq-marquee", MarqueeCallback, p_sys );
+    var_DelCallback( p_filter->p_libvlc, "marq-timeout", MarqueeCallback, p_sys );
+    var_DelCallback( p_filter->p_libvlc, "marq-position", MarqueeCallback, p_sys );
+    var_DelCallback( p_filter->p_libvlc, "marq-color", MarqueeCallback, p_sys );
+    var_DelCallback( p_filter->p_libvlc, "marq-opacity", MarqueeCallback, p_sys );
+    var_DelCallback( p_filter->p_libvlc, "marq-size", MarqueeCallback, p_sys );
 
-    var_Destroy( p_input->p_libvlc , "marq-marquee" );
-    var_Destroy( p_input->p_libvlc , "marq-x" );
-    var_Destroy( p_input->p_libvlc , "marq-y" );
-    var_Destroy( p_input->p_libvlc , "marq-timeout" );
-    var_Destroy( p_input->p_libvlc , "marq-position" );
-    var_Destroy( p_input->p_libvlc , "marq-color");
-    var_Destroy( p_input->p_libvlc , "marq-opacity");
-    var_Destroy( p_input->p_libvlc , "marq-size");
-
-    vlc_object_release( p_input );
+    var_Destroy( p_filter->p_libvlc , "marq-marquee" );
+    var_Destroy( p_filter->p_libvlc , "marq-x" );
+    var_Destroy( p_filter->p_libvlc , "marq-y" );
+    var_Destroy( p_filter->p_libvlc , "marq-timeout" );
+    var_Destroy( p_filter->p_libvlc , "marq-position" );
+    var_Destroy( p_filter->p_libvlc , "marq-color");
+    var_Destroy( p_filter->p_libvlc , "marq-opacity");
+    var_Destroy( p_filter->p_libvlc , "marq-size");
 }
 
 /****************************************************************************
@@ -293,10 +278,7 @@ static subpicture_t *Filter( filter_t *p_filter, mtime_t date )
         p_spu->i_y = p_sys->i_yoff;
         p_spu->b_absolute = VLC_TRUE;
     }
-    p_spu->p_region->i_text_color = p_sys->i_font_color;
-    p_spu->p_region->i_text_alpha = 255 - p_sys->i_font_opacity;
-    p_spu->p_region->i_text_size = p_sys->i_font_size;
-
+    p_spu->p_region->p_style = p_sys->p_style;
 
     p_sys->b_need_update = VLC_FALSE;
     return p_spu;
@@ -326,15 +308,15 @@ static int MarqueeCallback( vlc_object_t *p_this, char const *psz_var,
     }
     else if ( !strncmp( psz_var, "marq-color", 8 ) )  /* "marq-col" */
     {
-        p_sys->i_font_color = newval.i_int;
+        p_sys->p_style->i_font_color = newval.i_int;
     }
     else if ( !strncmp( psz_var, "marq-opacity", 8 ) ) /* "marq-opa" */
     {
-        p_sys->i_font_opacity = newval.i_int;
+        p_sys->p_style->i_font_alpha = 255 - newval.i_int;
     }
     else if ( !strncmp( psz_var, "marq-size", 6 ) )
     {
-        p_sys->i_font_size = newval.i_int;
+        p_sys->p_style->i_font_size = newval.i_int;
     }
     else if ( !strncmp( psz_var, "marq-timeout", 12 ) )
     {
