@@ -2,7 +2,7 @@
  * configuration.c management of the modules configuration
  *****************************************************************************
  * Copyright (C) 2001-2004 the VideoLAN team
- * $Id: configuration.c 12428 2005-08-29 16:34:32Z massiot $
+ * $Id: configuration.c 18163 2006-11-29 23:27:28Z xtophe $
  *
  * Authors: Gildas Bazin <gbazin@videolan.org>
  *
@@ -18,17 +18,22 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 #include <vlc/vlc.h>
 #include "vlc_keys.h"
+#include "charset.h"
 
 #include <stdio.h>                                              /* sprintf() */
 #include <stdlib.h>                                      /* free(), strtol() */
 #include <string.h>                                              /* strdup() */
 #include <errno.h>                                                  /* errno */
 
+#ifdef HAVE_LIMITS_H
+#   include <limits.h>
+#endif
+
 #ifdef HAVE_UNISTD_H
 #    include <unistd.h>                                          /* getuid() */
 #endif
@@ -58,6 +63,11 @@
 #include <tchar.h>
 #endif
 
+#if defined( WIN32 ) || defined( UNDER_CE )
+#   define DIR_SEP "\\"
+#else
+#   define DIR_SEP "/"
+#endif
 
 static int ConfigStringToKey( char * );
 static char *ConfigKeyToString( int );
@@ -770,7 +780,8 @@ int __config_LoadConfigFile( vlc_object_t *p_this, const char *psz_module_name )
         psz_filename = (char *)malloc( sizeof("/" CONFIG_DIR "/" CONFIG_FILE) +
                                        strlen(psz_homedir) );
         if( psz_filename )
-            sprintf( psz_filename, "%s/" CONFIG_DIR "/" CONFIG_FILE,
+            sprintf( psz_filename,
+                     "%s" DIR_SEP CONFIG_DIR DIR_SEP CONFIG_FILE,
                      psz_homedir );
     }
     else
@@ -789,7 +800,7 @@ int __config_LoadConfigFile( vlc_object_t *p_this, const char *psz_module_name )
     /* Acquire config file lock */
     vlc_mutex_lock( &p_this->p_vlc->config_lock );
 
-    file = fopen( psz_filename, "rt" );
+    file = utf8_fopen( psz_filename, "rt" );
     if( !file )
     {
         msg_Warn( p_this, "config file %s does not exist yet", psz_filename );
@@ -803,6 +814,9 @@ int __config_LoadConfigFile( vlc_object_t *p_this, const char *psz_module_name )
 
     for( i_index = 0; i_index < p_list->i_count; i_index++ )
     {
+        char * (*convert) (const char *) = FromLocaleDup;
+        char bom[3];
+
         p_parser = (module_t *)p_list->p_values[i_index].p_object ;
 
         if( psz_module_name
@@ -814,6 +828,15 @@ int __config_LoadConfigFile( vlc_object_t *p_this, const char *psz_module_name )
         /* The config file is organized in sections, one per module. Look for
          * the interesting section ( a section is of the form [foo] ) */
         fseek( file, 0L, SEEK_SET );
+
+        /* Look for UTF-8 Byte Order Mark */
+
+        if ((fread (bom, 1, 3, file) == 3)
+         && (memcmp (bom, "\xEF\xBB\xBF", 3) == 0))
+            convert = strdup;
+        else
+            rewind (file); // no BOM, rewind
+
         while( fgets( line, 1024, file ) )
         {
             if( (line[0] == '[')
@@ -889,7 +912,7 @@ int __config_LoadConfigFile( vlc_object_t *p_this, const char *psz_module_name )
                     case CONFIG_ITEM_FLOAT:
                         if( !*psz_option_value )
                             break;                    /* ignore empty option */
-                        p_item->f_value = (float)atof( psz_option_value);
+                        p_item->f_value = (float)i18n_atof( psz_option_value);
                         p_item->f_value_saved = p_item->f_value;
 #if 0
                         msg_Dbg( p_this, "option \"%s\", value %f",
@@ -911,7 +934,7 @@ int __config_LoadConfigFile( vlc_object_t *p_this, const char *psz_module_name )
                             free( p_item->psz_value );
 
                         p_item->psz_value = *psz_option_value ?
-                            strdup( psz_option_value ) : NULL;
+                            convert( psz_option_value ) : NULL;
 
                         if( p_item->psz_value_saved )
                             free( p_item->psz_value_saved );
@@ -950,46 +973,17 @@ int __config_LoadConfigFile( vlc_object_t *p_this, const char *psz_module_name )
 /*****************************************************************************
  * config_CreateDir: Create configuration directory if it doesn't exist.
  *****************************************************************************/
-int config_CreateDir( vlc_object_t *p_this, char *psz_dirname )
+int config_CreateDir( vlc_object_t *p_this, const char *psz_dirname )
 {
     if( !psz_dirname && !*psz_dirname ) return -1;
 
-#if defined( UNDER_CE )
-    {
-        wchar_t psz_new[ MAX_PATH ];
-        char psz_mod[MAX_PATH];
-        int i = 0;
-
-        /* Convert '/' into '\' */
-        while( *psz_dirname )
-        {
-            if( *psz_dirname == '/' ) psz_mod[i] = '\\';
-            else psz_mod[i] = *psz_dirname;
-            psz_dirname++;
-            i++;
-        }
-        psz_mod[i] = 0;
-
-        MultiByteToWideChar( CP_ACP, 0, psz_mod, -1, psz_new, MAX_PATH );
-        if( CreateDirectory( psz_new, NULL ) )
-        {
-            msg_Err( p_this, "could not create %s", psz_mod );
-        }
-    }
-
-#else
-#   if defined( WIN32 )
-    if( mkdir( psz_dirname ) && errno != EEXIST )
-#   else
-    if( mkdir( psz_dirname, 0755 ) && errno != EEXIST )
-#   endif
+    if( utf8_mkdir( psz_dirname ) && ( errno != EEXIST ) )
     {
         msg_Err( p_this, "could not create %s (%s)",
                  psz_dirname, strerror(errno) );
+        return -1;
     }
 
-#endif
-
     return 0;
 }
 
@@ -1043,7 +1037,7 @@ static int SaveConfigFile( vlc_object_t *p_this, const char *psz_module_name,
                                        strlen(psz_homedir) );
 
         if( psz_filename )
-            sprintf( psz_filename, "%s/" CONFIG_DIR, psz_homedir );
+            sprintf( psz_filename, "%s" DIR_SEP CONFIG_DIR, psz_homedir );
 
         if( !psz_filename )
         {
@@ -1054,7 +1048,7 @@ static int SaveConfigFile( vlc_object_t *p_this, const char *psz_module_name,
 
         config_CreateDir( p_this, psz_filename );
 
-        strcat( psz_filename, "/" CONFIG_FILE );
+        strcat( psz_filename, DIR_SEP CONFIG_FILE );
     }
     else
     {
@@ -1069,7 +1063,7 @@ static int SaveConfigFile( vlc_object_t *p_this, const char *psz_module_name,
 
     msg_Dbg( p_this, "opening config file %s", psz_filename );
 
-    file = fopen( psz_filename, "rt" );
+    file = utf8_fopen( psz_filename, "rt" );
     if( !file )
     {
         msg_Warn( p_this, "config file %s does not exist yet", psz_filename );
@@ -1157,7 +1151,7 @@ static int SaveConfigFile( vlc_object_t *p_this, const char *psz_module_name,
      * Save module config in file
      */
 
-    file = fopen( psz_filename, "wt" );
+    file = utf8_fopen( psz_filename, "wt" );
     if( !file )
     {
         msg_Warn( p_this, "could not open config file %s for writing",
@@ -1167,8 +1161,18 @@ static int SaveConfigFile( vlc_object_t *p_this, const char *psz_module_name,
         vlc_mutex_unlock( &p_this->p_vlc->config_lock );
         return -1;
     }
+    
+#ifdef WIN32
+    /* Ugly kludge to not save --started-from-file (and not break the ABI). 
+     * See [17898] and   #871 */
+    /* Just use the first mofule found*/
+    config_PutInt( (module_t *)p_list->p_values[0].p_object, 
+                   "started-from-file", 0 );
 
-    fprintf( file, "###\n###  " COPYRIGHT_MESSAGE "\n###\n\n" );
+#endif
+
+    fprintf( file, "\xEF\xBB\xBF###\n###  " COPYRIGHT_MESSAGE "\n###\n\n"
+       "###\n### lines begining with a '#' character are comments\n###\n\n" );
 
     /* Look for the selected module, if NULL then save everything */
     for( i_index = 0; i_index < p_list->i_count; i_index++ )
@@ -1228,7 +1232,7 @@ static int SaveConfigFile( vlc_object_t *p_this, const char *psz_module_name,
                              (p_item->i_type == CONFIG_ITEM_BOOL) ?
                              _("boolean") : _("integer") );
                 if( i_value == p_item->i_value_orig )
-                    fprintf( file, "#" );
+                    fputc( '#', file );
                 fprintf( file, "%s=%i\n", p_item->psz_name, i_value );
 
                 p_item->i_value_saved = i_value;
@@ -1239,7 +1243,7 @@ static int SaveConfigFile( vlc_object_t *p_this, const char *psz_module_name,
                     fprintf( file, "# %s (%s)\n", p_item->psz_text,
                              _("key") );
                 if( i_value == p_item->i_value_orig )
-                    fprintf( file, "#" );
+                    fputc( '#', file );
                 psz_key = ConfigKeyToString( i_value );
                 fprintf( file, "%s=%s\n", p_item->psz_name,
                          psz_key ? psz_key : "" );
@@ -1253,7 +1257,7 @@ static int SaveConfigFile( vlc_object_t *p_this, const char *psz_module_name,
                     fprintf( file, "# %s (%s)\n", p_item->psz_text,
                              _("float") );
                 if( f_value == p_item->f_value_orig )
-                    fprintf( file, "#" );
+                    fputc( '#', file );
                 fprintf( file, "%s=%f\n", p_item->psz_name, (double)f_value );
 
                 p_item->f_value_saved = f_value;
@@ -1266,7 +1270,7 @@ static int SaveConfigFile( vlc_object_t *p_this, const char *psz_module_name,
                 if( (!psz_value && !p_item->psz_value_orig) ||
                     (psz_value && p_item->psz_value_orig &&
                      !strcmp( psz_value, p_item->psz_value_orig )) )
-                    fprintf( file, "#" );
+                    fputc( '#', file );
                 fprintf( file, "%s=%s\n", p_item->psz_name,
                          psz_value ? psz_value : "" );
 
@@ -1281,7 +1285,7 @@ static int SaveConfigFile( vlc_object_t *p_this, const char *psz_module_name,
             }
         }
 
-        fprintf( file, "\n" );
+        fputc( '\n', file );
     }
 
     vlc_list_release( p_list );
@@ -1363,8 +1367,8 @@ int __config_LoadCmdLine( vlc_object_t *p_this, int *pi_argc, char *ppsz_argv[],
     p_this->p_vlc->i_argc    = *pi_argc;
     p_this->p_vlc->ppsz_argv = ppsz_argv;
 
-#ifdef SYS_DARWIN
-    /* When vlc.app is run by double clicking in Mac OS X, the 2nd arg
+#ifdef __APPLE__
+    /* When VLC.app is run by double clicking in Mac OS X, the 2nd arg
      * is the PSN - process serial number (a unique PID-ish thingie)
      * still ok for real Darwin & when run from command line */
     if ( (*pi_argc > 1) && (strncmp( ppsz_argv[ 1 ] , "-psn" , 4 ) == 0) )
@@ -1397,7 +1401,7 @@ int __config_LoadCmdLine( vlc_object_t *p_this, int *pi_argc, char *ppsz_argv[],
         p_parser = (module_t *)p_list->p_values[i_modules_index].p_object ;
 
         /* count the number of exported configuration options (to allocate
-         * longopts). We also need to allocate space for too options when
+         * longopts). We also need to allocate space for two options when
          * dealing with boolean to allow for --foo and --no-foo */
         i_opts += p_parser->i_config_items
                      + 2 * p_parser->i_bool_items;
@@ -1553,7 +1557,7 @@ int __config_LoadCmdLine( vlc_object_t *p_this, int *pi_argc, char *ppsz_argv[],
                 {
                     if( !strcmp(p_conf->psz_current,"SUPPRESSED") )
                     {
-                       if( !b_ignore_errors ) 
+                        if( !b_ignore_errors )
                         {
                             fprintf(stderr,
                                     "Warning: option --%s is no longer used.\n",
@@ -1586,83 +1590,83 @@ int __config_LoadCmdLine( vlc_object_t *p_this, int *pi_argc, char *ppsz_argv[],
                     p_conf = config_FindConfig( p_this, psz_name );
                 }
 
-            switch( p_conf->i_type )
+                switch( p_conf->i_type )
+                {
+                    case CONFIG_ITEM_STRING:
+                    case CONFIG_ITEM_FILE:
+                    case CONFIG_ITEM_DIRECTORY:
+                    case CONFIG_ITEM_MODULE:
+                    case CONFIG_ITEM_MODULE_LIST:
+                    case CONFIG_ITEM_MODULE_LIST_CAT:
+                    case CONFIG_ITEM_MODULE_CAT:
+                        config_PutPsz( p_this, psz_name, optarg );
+                        break;
+                    case CONFIG_ITEM_INTEGER:
+                        config_PutInt( p_this, psz_name, strtol(optarg, 0, 0));
+                        break;
+                    case CONFIG_ITEM_FLOAT:
+                        config_PutFloat( p_this, psz_name, (float)atof(optarg) );
+                        break;
+                    case CONFIG_ITEM_KEY:
+                        config_PutInt( p_this, psz_name, ConfigStringToKey( optarg ) );
+                        break;
+                    case CONFIG_ITEM_BOOL:
+                        config_PutInt( p_this, psz_name, !flag );
+                        break;
+                }
+                continue;
+            }
+        }
+
+        /* A short option has been recognized */
+        if( pp_shortopts[i_cmd] != NULL )
+        {
+            switch( pp_shortopts[i_cmd]->i_type )
             {
                 case CONFIG_ITEM_STRING:
                 case CONFIG_ITEM_FILE:
                 case CONFIG_ITEM_DIRECTORY:
                 case CONFIG_ITEM_MODULE:
+                case CONFIG_ITEM_MODULE_CAT:
                 case CONFIG_ITEM_MODULE_LIST:
                 case CONFIG_ITEM_MODULE_LIST_CAT:
-                case CONFIG_ITEM_MODULE_CAT:
-                    config_PutPsz( p_this, psz_name, optarg );
+                    config_PutPsz( p_this, pp_shortopts[i_cmd]->psz_name, optarg );
                     break;
                 case CONFIG_ITEM_INTEGER:
-                    config_PutInt( p_this, psz_name, strtol(optarg, 0, 0));
-                    break;
-                case CONFIG_ITEM_FLOAT:
-                    config_PutFloat( p_this, psz_name, (float)atof(optarg) );
-                    break;
-                case CONFIG_ITEM_KEY:
-                    config_PutInt( p_this, psz_name, ConfigStringToKey( optarg ) );
-                    break;
-                case CONFIG_ITEM_BOOL:
-                    config_PutInt( p_this, psz_name, !flag );
-                    break;
-            }
-
-            continue;
-        }
-    }
-    /* A short option has been recognized */
-    if( pp_shortopts[i_cmd] != NULL )
-    {
-        switch( pp_shortopts[i_cmd]->i_type )
-        {
-            case CONFIG_ITEM_STRING:
-            case CONFIG_ITEM_FILE:
-            case CONFIG_ITEM_DIRECTORY:
-            case CONFIG_ITEM_MODULE:
-            case CONFIG_ITEM_MODULE_CAT:
-            case CONFIG_ITEM_MODULE_LIST:
-            case CONFIG_ITEM_MODULE_LIST_CAT:
-                config_PutPsz( p_this, pp_shortopts[i_cmd]->psz_name, optarg );
-                break;
-            case CONFIG_ITEM_INTEGER:
-                if( i_cmd == 'v' )
-                {
-                    if( optarg )
+                    if( i_cmd == 'v' )
                     {
-                        if( *optarg == 'v' ) /* eg. -vvv */
+                        if( optarg )
                         {
-                            i_verbose++;
-                            while( *optarg == 'v' )
+                            if( *optarg == 'v' ) /* eg. -vvv */
                             {
                                 i_verbose++;
-                                optarg++;
+                                while( *optarg == 'v' )
+                                {
+                                    i_verbose++;
+                                    optarg++;
+                                }
+                            }
+                            else
+                            {
+                                i_verbose += atoi( optarg ); /* eg. -v2 */
                             }
                         }
                         else
                         {
-                            i_verbose += atoi( optarg ); /* eg. -v2 */
+                            i_verbose++; /* -v */
                         }
+                        config_PutInt( p_this, pp_shortopts[i_cmd]->psz_name,
+                                               i_verbose );
                     }
                     else
                     {
-                        i_verbose++; /* -v */
+                        config_PutInt( p_this, pp_shortopts[i_cmd]->psz_name,
+                                               strtol(optarg, 0, 0) );
                     }
-                    config_PutInt( p_this, pp_shortopts[i_cmd]->psz_name,
-                                           i_verbose );
-                }
-                else
-                {
-                    config_PutInt( p_this, pp_shortopts[i_cmd]->psz_name,
-                                           strtol(optarg, 0, 0) );
-                }
-                break;
-            case CONFIG_ITEM_BOOL:
-                config_PutInt( p_this, pp_shortopts[i_cmd]->psz_name, 1 );
-                break;
+                    break;
+                case CONFIG_ITEM_BOOL:
+                    config_PutInt( p_this, pp_shortopts[i_cmd]->psz_name, 1 );
+                    break;
             }
 
             continue;
@@ -1703,6 +1707,30 @@ int __config_LoadCmdLine( vlc_object_t *p_this, int *pi_argc, char *ppsz_argv[],
     return 0;
 }
 
+/**
+ * config_GetDataDir: find directory where shared data is installed
+ *
+ * @return a string (always succeeds).
+ */
+const char *config_GetDataDir( const vlc_object_t *p_this )
+{
+#if defined (WIN32) || defined (UNDER_CE)
+    return p_this->p_libvlc->psz_vlcpath;
+#elif defined(__APPLE__) || defined (SYS_BEOS)
+    static char path[PATH_MAX] = "";
+
+    if( *path == '\0' )
+    {
+        snprintf( path, sizeof( path ), "%s/share",
+                  p_this->p_libvlc->psz_vlcpath );
+        path[sizeof( path ) - 1] = '\0';
+    }
+    return path;
+#else
+    return DATA_PATH;
+#endif
+}
+
 /*****************************************************************************
  * config_GetHomeDir, config_GetUserDir: find the user's home directory.
  *****************************************************************************
@@ -1712,7 +1740,7 @@ int __config_LoadCmdLine( vlc_object_t *p_this, int *pi_argc, char *ppsz_argv[],
  *****************************************************************************/
 static char *GetDir( vlc_bool_t b_appdata )
 {
-    char *p_tmp, *p_homedir = NULL;
+    char *psz_localhome = NULL;
 
 #if defined(HAVE_GETPWUID)
     struct passwd *p_pw = NULL;
@@ -1744,21 +1772,18 @@ static char *GetDir( vlc_bool_t b_appdata )
                                                   _T("SHGetFolderPathA") );
         if ( SHGetFolderPath != NULL )
         {
-            p_homedir = (char *)malloc( MAX_PATH );
-            if( !p_homedir ) return NULL;
+            char psz_ACPhome[MAX_PATH];
 
             /* get the "Application Data" folder for the current user */
             if( S_OK == SHGetFolderPath( NULL,
                                          (b_appdata ? CSIDL_APPDATA :
                                            CSIDL_PROFILE) | CSIDL_FLAG_CREATE,
                                          NULL, SHGFP_TYPE_CURRENT,
-                                         p_homedir ) )
+                                         psz_ACPhome ) )
             {
                 FreeLibrary( shfolder_dll );
-                return p_homedir;
+                return FromLocaleDup( psz_ACPhome );
             }
-            free( p_homedir );
-            p_homedir = NULL;
         }
         FreeLibrary( shfolder_dll );
     }
@@ -1774,11 +1799,10 @@ static char *GetDir( vlc_bool_t b_appdata )
     /* get the "Application Data" folder for the current user */
     if( SHGetSpecialFolderPath( NULL, p_whomedir, CSIDL_APPDATA, 1 ) )
     {
-        p_homedir = (char *)malloc( MAX_PATH );
-        if( !p_homedir ) return NULL;
+        char psz_ACPhome[2 * MAX_PATH];
 
-        sprintf( p_homedir, "%ls", p_whomedir );
-        return p_homedir;
+        sprintf( psz_ACPhome, "%ls", p_whomedir );
+        return FromLocaleDup( psz_ACPhome );
     }
 #endif
 
@@ -1786,24 +1810,20 @@ static char *GetDir( vlc_bool_t b_appdata )
     if( ( p_pw = getpwuid( getuid() ) ) == NULL )
 #endif
     {
-        if( ( p_tmp = getenv( "HOME" ) ) == NULL )
+        psz_localhome = getenv( "HOME" );
+        if( psz_localhome == NULL )
         {
-            if( ( p_tmp = getenv( "TMP" ) ) == NULL )
-            {
-                p_tmp = "/tmp";
-            }
+            psz_localhome = getenv( "TMP" );
+            if( psz_localhome == NULL )
+                psz_localhome = "/tmp";
         }
-
-        p_homedir = strdup( p_tmp );
     }
 #if defined(HAVE_GETPWUID)
     else
-    {
-        p_homedir = strdup( p_pw->pw_dir );
-    }
+        psz_localhome = p_pw->pw_dir;
 #endif
 
-    return p_homedir;
+    return FromLocaleDup( psz_localhome );
 }
 
 char *config_GetHomeDir( void )
