@@ -2,7 +2,7 @@
  * darwin_specific.m: Darwin specific features
  *****************************************************************************
  * Copyright (C) 2001-2004 the VideoLAN team
- * $Id: darwin_specific.m 15281 2006-04-19 14:24:55Z fkuehne $
+ * $Id: darwin_specific.m 19472 2007-03-25 19:49:02Z fkuehne $
  *
  * Authors: Sam Hocevar <sam@zoy.org>
  *          Christophe Massiot <massiot@via.ecp.fr>
@@ -42,6 +42,7 @@ static int FindLanguage( const char * psz_lang )
     const char * ppsz_all[] =
     {
         "Catalan", "ca",
+	"Czech", "cs",
         "Danish", "da",
         "German", "de",
         "British", "en_GB",
@@ -56,12 +57,17 @@ static int FindLanguage( const char * psz_lang )
         "Japanese", "ja",
         "Korean", "ko",
         "Georgian", "ka",
-        "Dutch", "nl",
+        "Malay", "ms",
+	"Nepali", "ne",
+	"Dutch", "nl",
         "Occitan", "oc",
         "Brazilian Portuguese", "pt_BR",
         "Romanian", "ro",
         "Russian", "ru",
+        "Slovak", "sk",
+        "Slovenian", "sl",
         "Swedish", "sv",
+	"Thai", "th",
         "Turkish", "tr",
         "Simplified Chinese", "zh_CN",
         "Chinese Traditional", "zh_TW",
