@@ -6,7 +6,7 @@
  * thread, and destroy a previously oppened video output thread.
  *****************************************************************************
  * Copyright (C) 2000-2004 the VideoLAN team
- * $Id: video_output.c 15328 2006-04-23 13:56:24Z bigben $
+ * $Id: video_output.c 17428 2006-11-02 20:43:55Z hartman $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *          Gildas Bazin <gbazin@videolan.org>
@@ -470,9 +470,14 @@ void vout_Destroy( vout_thread_t *p_vout )
     /* Free structure */
     vlc_object_destroy( p_vout );
 
+    /* This is a dirty hack for mostly Linux, where there is no way to get the GUI
+       back if you closed it while playing video. This is solved in Mac OS X,
+       where we have this novelty called menubar, that will always allow you access
+       to the applications main functionality. They should try that on linux sometime */
     /* If it was the last vout, tell the interface to show up */
     if( p_playlist != NULL )
     {
+#ifndef __APPLE__
         vout_thread_t *p_another_vout = vlc_object_find( p_playlist,
                                             VLC_OBJECT_VOUT, FIND_ANYWHERE );
         if( p_another_vout == NULL )
@@ -485,6 +490,7 @@ void vout_Destroy( vout_thread_t *p_vout )
         {
             vlc_object_release( p_another_vout );
         }
+#endif
         vlc_object_release( p_playlist );
     }
 }
@@ -701,11 +707,20 @@ static void RunThread( vout_thread_t *p_vout)
 
     subpicture_t *  p_subpic = NULL;                   /* subpicture pointer */
 
+    input_thread_t *p_input = NULL ;           /* Parent input, if it exists */
+
+    vlc_value_t     val;
+    vlc_bool_t      b_drop_late;
+
     /*
      * Initialize thread
      */
     p_vout->b_error = InitThread( p_vout );
 
+    var_Create( p_vout, "drop-late-frames", VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
+    var_Get( p_vout, "drop-late-frames", &val );
+    b_drop_late = val.b_bool;
+
     /* signal the creation of the vout */
     vlc_thread_ready( p_vout );
 
@@ -727,6 +742,12 @@ static void RunThread( vout_thread_t *p_vout)
         display_date = 0;
         current_date = mdate();
 
+        if( p_input && p_input->b_die )
+        {
+            vlc_object_release( p_input );
+            p_input = NULL;
+        }
+
 #if 0
         p_vout->c_loops++;
         if( !(p_vout->c_loops % VOUT_STATS_NB_LOOPS) )
@@ -791,12 +812,10 @@ static void RunThread( vout_thread_t *p_vout)
             p_vout->p_fps_sample[ p_vout->c_fps_samples++ % VOUT_FPS_SAMPLES ]
                 = display_date;
 
-            /* XXX: config_GetInt is slow, but this kind of frame dropping
-             * should not happen that often. */
             if( !p_picture->b_force &&
                 p_picture != p_last_picture &&
                 display_date < current_date + p_vout->render_time &&
-                config_GetInt( p_vout, "drop-late-frames" ) )
+                b_drop_late )
             {
                 /* Picture is late: it will be destroyed and the thread
                  * will directly choose the next picture */
@@ -890,7 +909,11 @@ static void RunThread( vout_thread_t *p_vout)
          */
         if( display_date > 0 )
         {
-            p_subpic = spu_SortSubpictures( p_vout->p_spu, display_date );
+            if( !p_input )
+                p_input = vlc_object_find( p_vout, VLC_OBJECT_INPUT,
+                                           FIND_PARENT );
+            p_subpic = spu_SortSubpictures( p_vout->p_spu, display_date,
+                p_input ? var_GetBool( p_input, "state" ) == PAUSE_S : VLC_FALSE );
         }
 
         /*
@@ -1039,6 +1062,11 @@ static void RunThread( vout_thread_t *p_vout)
         }
     }
 
+    if( p_input )
+    {
+        vlc_object_release( p_input );
+    }
+
     /*
      * Error loop - wait until the thread destruction is requested
      */
