@@ -1,8 +1,8 @@
 /*****************************************************************************
  * vout_intf.c : video output interface
  *****************************************************************************
- * Copyright (C) 2000-2004 the VideoLAN team
- * $Id: vout_intf.c 13051 2005-10-31 07:35:39Z md $
+ * Copyright (C) 2000-2006 the VideoLAN team
+ * $Id: vout_intf.c 18330 2006-12-08 18:25:54Z hartman $
  *
  * Authors: Gildas Bazin <gbazin@videolan.org>
  *
@@ -18,21 +18,27 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
  * Preamble
  *****************************************************************************/
 #include <stdlib.h>                                                /* free() */
+#include <sys/types.h>                                          /* opendir() */
+#include <dirent.h>                                             /* opendir() */
 
 #include <vlc/vlc.h>
 #include <vlc/intf.h>
+#include <vlc_block.h>
 
 #include "vlc_video.h"
 #include "video_output.h"
 #include "vlc_image.h"
 #include "vlc_spu.h"
+#include "charset.h"
+
+#include <snapshot.h>
 
 /*****************************************************************************
  * Local prototypes
@@ -178,10 +184,18 @@ void vout_IntfInit( vout_thread_t *p_vout )
 {
     vlc_value_t val, text, old_val;
     vlc_bool_t b_force_par = VLC_FALSE;
+    char *psz_buf;
 
     /* Create a few object variables we'll need later on */
     var_Create( p_vout, "snapshot-path", VLC_VAR_STRING | VLC_VAR_DOINHERIT );
+    var_Create( p_vout, "snapshot-prefix", VLC_VAR_STRING | VLC_VAR_DOINHERIT );
     var_Create( p_vout, "snapshot-format", VLC_VAR_STRING | VLC_VAR_DOINHERIT );
+    var_Create( p_vout, "snapshot-preview", VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
+    var_Create( p_vout, "snapshot-sequential",
+                VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
+    var_Create( p_vout, "snapshot-num", VLC_VAR_INTEGER );
+    var_SetInteger( p_vout, "snapshot-num", 1 );
+
     var_Create( p_vout, "width", VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
     var_Create( p_vout, "height", VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
     var_Create( p_vout, "align", VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
@@ -220,6 +234,22 @@ void vout_IntfInit( vout_thread_t *p_vout )
 
     var_AddCallback( p_vout, "zoom", ZoomCallback, NULL );
 
+    /* Crop offset vars */
+    var_Create( p_vout, "crop-left", VLC_VAR_INTEGER );
+    var_Create( p_vout, "crop-top", VLC_VAR_INTEGER );
+    var_Create( p_vout, "crop-right", VLC_VAR_INTEGER );
+    var_Create( p_vout, "crop-bottom", VLC_VAR_INTEGER );
+
+    var_SetInteger( p_vout, "crop-left", 0 );
+    var_SetInteger( p_vout, "crop-top", 0 );
+    var_SetInteger( p_vout, "crop-right", 0 );
+    var_SetInteger( p_vout, "crop-bottom", 0 );
+
+    var_AddCallback( p_vout, "crop-left", CropCallback, NULL );
+    var_AddCallback( p_vout, "crop-top", CropCallback, NULL );
+    var_AddCallback( p_vout, "crop-right", CropCallback, NULL );
+    var_AddCallback( p_vout, "crop-bottom", CropCallback, NULL );
+
     /* Crop object var */
     var_Create( p_vout, "crop", VLC_VAR_STRING |
                 VLC_VAR_HASCHOICE | VLC_VAR_DOINHERIT );
@@ -231,14 +261,42 @@ void vout_IntfInit( vout_thread_t *p_vout )
     var_Change( p_vout, "crop", VLC_VAR_DELCHOICE, &val, 0 );
     val.psz_string = ""; text.psz_string = _("Default");
     var_Change( p_vout, "crop", VLC_VAR_ADDCHOICE, &val, &text );
-    val.psz_string = "001:1"; text.psz_string = _("1:1");
+    val.psz_string = "1:1"; text.psz_string = "1:1";
     var_Change( p_vout, "crop", VLC_VAR_ADDCHOICE, &val, &text );
-    val.psz_string = "004:3"; text.psz_string = _("4:3");
+    val.psz_string = "4:3"; text.psz_string = "4:3";
     var_Change( p_vout, "crop", VLC_VAR_ADDCHOICE, &val, &text );
-    val.psz_string = "16:9"; text.psz_string = _("16:9");
+    val.psz_string = "16:9"; text.psz_string = "16:9";
     var_Change( p_vout, "crop", VLC_VAR_ADDCHOICE, &val, &text );
-    val.psz_string = "221:100"; text.psz_string = _("221:100");
+    val.psz_string = "16:10"; text.psz_string = "16:10";
     var_Change( p_vout, "crop", VLC_VAR_ADDCHOICE, &val, &text );
+    val.psz_string = "221:100"; text.psz_string = "221:100";
+    var_Change( p_vout, "crop", VLC_VAR_ADDCHOICE, &val, &text );
+    val.psz_string = "5:4"; text.psz_string = "5:4";
+    var_Change( p_vout, "crop", VLC_VAR_ADDCHOICE, &val, &text );
+
+    /* Add custom crop ratios */
+    psz_buf = config_GetPsz( p_vout, "custom-crop-ratios" );
+    if( psz_buf && *psz_buf )
+    {
+        char *psz_cur = psz_buf;
+        char *psz_next;
+        while( psz_cur && *psz_cur )
+        {
+            psz_next = strchr( psz_cur, ',' );
+            if( psz_next )
+            {
+                *psz_next = '\0';
+                psz_next++;
+            }
+            val.psz_string = strdup( psz_cur );
+            text.psz_string = strdup( psz_cur );
+            var_Change( p_vout, "crop", VLC_VAR_ADDCHOICE, &val, &text);
+            free( val.psz_string );
+            free( text.psz_string );
+            psz_cur = psz_next;
+        }
+    }
+    if( psz_buf ) free( psz_buf );
 
     var_AddCallback( p_vout, "crop", CropCallback, NULL );
     var_Get( p_vout, "crop", &old_val );
@@ -256,8 +314,8 @@ void vout_IntfInit( vout_thread_t *p_vout )
         float i_aspect = 0;
         if( psz_parser )
         {
-            i_aspect_num = strtol( val.psz_string, 0, 0 );
-            i_aspect_den = strtol( ++psz_parser, 0, 0 );
+            i_aspect_num = strtol( val.psz_string, 0, 10 );
+            i_aspect_den = strtol( ++psz_parser, 0, 10 );
         }
         else
         {
@@ -273,7 +331,7 @@ void vout_IntfInit( vout_thread_t *p_vout )
         vlc_ureduce( &p_vout->i_par_num, &p_vout->i_par_den,
                      p_vout->i_par_num, p_vout->i_par_den, 0 );
 
-        msg_Dbg( p_vout, "monitor pixel aspect-ratio overriding: %i:%i",
+        msg_Dbg( p_vout, "overriding monitor pixel aspect-ratio: %i:%i",
                  p_vout->i_par_num, p_vout->i_par_den );
         b_force_par = VLC_TRUE;
     }
@@ -290,14 +348,42 @@ void vout_IntfInit( vout_thread_t *p_vout )
     var_Change( p_vout, "aspect-ratio", VLC_VAR_DELCHOICE, &val, 0 );
     val.psz_string = ""; text.psz_string = _("Default");
     var_Change( p_vout, "aspect-ratio", VLC_VAR_ADDCHOICE, &val, &text );
-    val.psz_string = "001:1"; text.psz_string = _("1:1");
+    val.psz_string = "1:1"; text.psz_string = "1:1";
     var_Change( p_vout, "aspect-ratio", VLC_VAR_ADDCHOICE, &val, &text );
-    val.psz_string = "004:3"; text.psz_string = _("4:3");
+    val.psz_string = "4:3"; text.psz_string = "4:3";
     var_Change( p_vout, "aspect-ratio", VLC_VAR_ADDCHOICE, &val, &text );
-    val.psz_string = "16:9"; text.psz_string = _("16:9");
+    val.psz_string = "16:9"; text.psz_string = "16:9";
     var_Change( p_vout, "aspect-ratio", VLC_VAR_ADDCHOICE, &val, &text );
-    val.psz_string = "221:100"; text.psz_string = _("221:100");
+    val.psz_string = "16:10"; text.psz_string = "16:10";
     var_Change( p_vout, "aspect-ratio", VLC_VAR_ADDCHOICE, &val, &text );
+    val.psz_string = "221:100"; text.psz_string = "221:100";
+    var_Change( p_vout, "aspect-ratio", VLC_VAR_ADDCHOICE, &val, &text );
+    val.psz_string = "5:4"; text.psz_string = "5:4";
+    var_Change( p_vout, "aspect-ratio", VLC_VAR_ADDCHOICE, &val, &text );
+
+    /* Add custom aspect ratios */
+    psz_buf = config_GetPsz( p_vout, "custom-aspect-ratios" );
+    if( psz_buf && *psz_buf )
+    {
+        char *psz_cur = psz_buf;
+        char *psz_next;
+        while( psz_cur && *psz_cur )
+        {
+            psz_next = strchr( psz_cur, ',' );
+            if( psz_next )
+            {
+                *psz_next = '\0';
+                psz_next++;
+            }
+            val.psz_string = strdup( psz_cur );
+            text.psz_string = strdup( psz_cur );
+            var_Change( p_vout, "aspect-ratio", VLC_VAR_ADDCHOICE, &val, &text);
+            free( val.psz_string );
+            free( text.psz_string );
+            psz_cur = psz_next;
+        }
+    }
+    if( psz_buf ) free( psz_buf );
 
     var_AddCallback( p_vout, "aspect-ratio", AspectCallback, NULL );
     var_Get( p_vout, "aspect-ratio", &old_val );
@@ -355,10 +441,12 @@ int vout_Snapshot( vout_thread_t *p_vout, picture_t *p_pic )
 {
     image_handler_t *p_image = image_HandlerCreate( p_vout );
     video_format_t fmt_in = {0}, fmt_out = {0};
-    char *psz_filename;
+    char *psz_filename = NULL;
     subpicture_t *p_subpic;
     picture_t *p_pif;
     vlc_value_t val, format;
+    DIR *path;
+
     int i_ret;
 
     var_Get( p_vout, "snapshot-path", &val );
@@ -368,7 +456,98 @@ int vout_Snapshot( vout_thread_t *p_vout, picture_t *p_pic )
         val.psz_string = 0;
     }
 
-#if defined(SYS_DARWIN) || defined(SYS_BEOS)
+    /* Embedded snapshot : if snapshot-path == object:object-id, then
+       create a snapshot_t* and store it in
+       object(object-id)->p_private, then unlock and signal the
+       waiting object.
+     */
+    if( val.psz_string && !strncmp( val.psz_string, "object:", 7 ) )
+    {
+        int i_id;
+        vlc_object_t* p_dest;
+        block_t *p_block;
+        snapshot_t *p_snapshot;
+        int i_size;
+
+        /* Destination object-id is following object: */
+        i_id = atoi( &val.psz_string[7] );
+        p_dest = ( vlc_object_t* )vlc_current_object( i_id );
+        if( !p_dest )
+        {
+            msg_Err( p_vout, "Cannot find calling object" );
+            image_HandlerDelete( p_image );
+            return VLC_EGENERIC;
+        }
+        /* Object must be locked. We will unlock it once we get the
+           snapshot and written it to p_private */
+        p_dest->p_private = NULL;
+
+        /* Save the snapshot to a memory zone */
+        fmt_in = p_vout->fmt_in;
+        fmt_out.i_sar_num = fmt_out.i_sar_den = 1;
+        /* FIXME: should not be hardcoded. We should be able to
+        specify the snapshot size (snapshot-width and snapshot-height). */
+        fmt_out.i_width = 320;
+        fmt_out.i_height = 200;
+        fmt_out.i_chroma = VLC_FOURCC( 'p','n','g',' ' );
+        p_block = ( block_t* ) image_Write( p_image, p_pic, &fmt_in, &fmt_out );
+        if( !p_block )
+        {
+            msg_Err( p_vout, "Could not get snapshot" );
+            image_HandlerDelete( p_image );
+            vlc_cond_signal( &p_dest->object_wait );
+            vlc_mutex_unlock( &p_dest->object_lock );
+            vlc_object_release( p_dest );
+            return VLC_EGENERIC;
+        }
+
+        /* Copy the p_block data to a snapshot structure */
+        /* FIXME: get the timestamp */
+        p_snapshot = ( snapshot_t* ) malloc( sizeof( snapshot_t ) );
+        if( !p_snapshot )
+        {
+            block_Release( p_block );
+            image_HandlerDelete( p_image );
+            vlc_cond_signal( &p_dest->object_wait );
+            vlc_mutex_unlock( &p_dest->object_lock );
+            vlc_object_release( p_dest );
+            return VLC_ENOMEM;
+        }
+
+        i_size = p_block->i_buffer;
+
+        p_snapshot->i_width = fmt_out.i_width;
+        p_snapshot->i_height = fmt_out.i_height;
+        p_snapshot->i_datasize = i_size;
+        p_snapshot->date = p_block->i_pts; /* FIXME ?? */
+        p_snapshot->p_data = ( char* ) malloc( i_size );
+        if( !p_snapshot->p_data )
+        {
+            block_Release( p_block );
+            free( p_snapshot );
+            image_HandlerDelete( p_image );
+            vlc_cond_signal( &p_dest->object_wait );
+            vlc_mutex_unlock( &p_dest->object_lock );
+            vlc_object_release( p_dest );
+            return VLC_ENOMEM;
+        }
+        memcpy( p_snapshot->p_data, p_block->p_buffer, p_block->i_buffer );
+
+        p_dest->p_private = p_snapshot;
+
+        block_Release( p_block );
+
+        /* Unlock the object */
+        vlc_cond_signal( &p_dest->object_wait );
+        vlc_mutex_unlock( &p_dest->object_lock );
+        vlc_object_release( p_dest );
+
+        image_HandlerDelete( p_image );
+        return VLC_SUCCESS;
+    }
+
+
+#if defined(__APPLE__) || defined(SYS_BEOS)
     if( !val.psz_string && p_vout->p_vlc->psz_homedir )
     {
         asprintf( &val.psz_string, "%s/Desktop",
@@ -442,7 +621,7 @@ int vout_Snapshot( vout_thread_t *p_vout, picture_t *p_pic )
 
     if( !val.psz_string )
     {
-        msg_Err( p_vout, "no directory specified for snapshots" );
+        msg_Err( p_vout, "no path specified for snapshots" );
         return VLC_EGENERIC;
     }
     var_Get( p_vout, "snapshot-format", &format );
@@ -452,9 +631,44 @@ int vout_Snapshot( vout_thread_t *p_vout, picture_t *p_pic )
         format.psz_string = strdup( "png" );
     }
 
-    asprintf( &psz_filename, "%s/vlcsnap-%u.%s", val.psz_string,
-              (unsigned int)(p_pic->date / 100000) & 0xFFFFFF,
-              format.psz_string );
+    /*
+     * Did the user specify a directory? If not, path = NULL.
+     */
+    path = utf8_opendir ( (const char *)val.psz_string  );
+
+    if ( path != NULL )
+    {
+        char *psz_prefix = var_GetString( p_vout, "snapshot-prefix" );
+        if( !psz_prefix ) psz_prefix = strdup( "vlcsnap-" );
+
+        vlc_closedir_wrapper( path );
+        if( var_GetBool( p_vout, "snapshot-sequential" ) == VLC_TRUE )
+        {
+            int i_num = var_GetInteger( p_vout, "snapshot-num" );
+            FILE *p_file;
+            do
+            {
+                asprintf( &psz_filename, "%s/%s%05d.%s", val.psz_string,
+                          psz_prefix, i_num++, format.psz_string );
+            }
+            while( ( p_file = utf8_fopen( psz_filename, "r" ) ) && !fclose( p_file ) );
+            var_SetInteger( p_vout, "snapshot-num", i_num );
+        }
+        else
+        {
+            asprintf( &psz_filename, "%s/%s%u.%s", val.psz_string,
+                      psz_prefix,
+                      (unsigned int)(p_pic->date / 100000) & 0xFFFFFF,
+                      format.psz_string );
+        }
+
+        free( psz_prefix );
+    }
+    else // The user specified a full path name (including file name)
+    {
+        asprintf ( &psz_filename, "%s", val.psz_string );
+    }
+
     free( val.psz_string );
     free( format.psz_string );
 
@@ -471,35 +685,45 @@ int vout_Snapshot( vout_thread_t *p_vout, picture_t *p_pic )
     }
 
     msg_Dbg( p_vout, "snapshot taken (%s)", psz_filename );
+    vout_OSDMessage( VLC_OBJECT( p_vout ), DEFAULT_CHAN,
+                     "%s", psz_filename );
     free( psz_filename );
 
-    /* Inject a subpicture with the snapshot */
-    memset( &fmt_out, 0, sizeof(fmt_out) );
-    fmt_out.i_chroma = VLC_FOURCC('Y','U','V','A');
-    p_pif = image_Convert( p_image, p_pic, &fmt_in, &fmt_out );
-    image_HandlerDelete( p_image );
-    if( !p_pif ) return VLC_EGENERIC;
-
-    p_subpic = spu_CreateSubpicture( p_vout->p_spu );
-    if( p_subpic == NULL )
+    if( var_GetBool( p_vout, "snapshot-preview" ) )
     {
-         p_pif->pf_release( p_pif );
-         return VLC_EGENERIC;
+        /* Inject a subpicture with the snapshot */
+        memset( &fmt_out, 0, sizeof(fmt_out) );
+        fmt_out.i_chroma = VLC_FOURCC('Y','U','V','A');
+        p_pif = image_Convert( p_image, p_pic, &fmt_in, &fmt_out );
+        image_HandlerDelete( p_image );
+        if( !p_pif ) return VLC_EGENERIC;
+
+        p_subpic = spu_CreateSubpicture( p_vout->p_spu );
+        if( p_subpic == NULL )
+        {
+             p_pif->pf_release( p_pif );
+             return VLC_EGENERIC;
+        }
+
+        p_subpic->i_channel = 0;
+        p_subpic->i_start = mdate();
+        p_subpic->i_stop = mdate() + 4000000;
+        p_subpic->b_ephemer = VLC_TRUE;
+        p_subpic->b_fade = VLC_TRUE;
+        p_subpic->i_original_picture_width = p_vout->render.i_width * 4;
+        p_subpic->i_original_picture_height = p_vout->render.i_height * 4;
+
+        p_subpic->p_region = spu_CreateRegion( p_vout->p_spu, &fmt_out );
+        vout_CopyPicture( p_image->p_parent, &p_subpic->p_region->picture,
+                          p_pif );
+        p_pif->pf_release( p_pif );
+
+        spu_DisplaySubpicture( p_vout->p_spu, p_subpic );
+    }
+    else
+    {
+        image_HandlerDelete( p_image );
     }
-
-    p_subpic->i_channel = 0;
-    p_subpic->i_start = mdate();
-    p_subpic->i_stop = mdate() + 4000000;
-    p_subpic->b_ephemer = VLC_TRUE;
-    p_subpic->b_fade = VLC_TRUE;
-    p_subpic->i_original_picture_width = p_vout->render.i_width * 4;
-    p_subpic->i_original_picture_height = p_vout->render.i_height * 4;
-
-    p_subpic->p_region = spu_CreateRegion( p_vout->p_spu, &fmt_out );
-    vout_CopyPicture( p_image->p_parent, &p_subpic->p_region->picture, p_pif );
-    p_pif->pf_release( p_pif );
-
-    spu_DisplaySubpicture( p_vout->p_spu, p_subpic );
 
     return VLC_SUCCESS;
 }
@@ -608,7 +832,8 @@ static int ZoomCallback( vlc_object_t *p_this, char const *psz_cmd,
     vout_thread_t *p_vout = (vout_thread_t *)p_this;
     InitWindowSize( p_vout, &p_vout->i_window_width,
                     &p_vout->i_window_height );
-    vout_Control( p_vout, VOUT_SET_ZOOM );
+    vout_Control( p_vout, VOUT_SET_SIZE, p_vout->i_window_width,
+                  p_vout->i_window_height );
     return VLC_SUCCESS;
 }
 
@@ -619,38 +844,130 @@ static int CropCallback( vlc_object_t *p_this, char const *psz_cmd,
     int64_t i_aspect_num, i_aspect_den;
     unsigned int i_width, i_height;
 
-    char *psz_end, *psz_parser = strchr( newval.psz_string, ':' );
-
     /* Restore defaults */
     p_vout->fmt_in.i_x_offset = p_vout->fmt_render.i_x_offset;
     p_vout->fmt_in.i_visible_width = p_vout->fmt_render.i_visible_width;
     p_vout->fmt_in.i_y_offset = p_vout->fmt_render.i_y_offset;
     p_vout->fmt_in.i_visible_height = p_vout->fmt_render.i_visible_height;
 
-    if( !psz_parser ) goto crop_end;
-
-    i_aspect_num = strtol( newval.psz_string, &psz_end, 0 );
-    if( psz_end == newval.psz_string || !i_aspect_num ) goto crop_end;
-
-    i_aspect_den = strtol( ++psz_parser, &psz_end, 0 );
-    if( psz_end == psz_parser || !i_aspect_den ) goto crop_end;
-
-    i_width = p_vout->fmt_in.i_sar_den * p_vout->fmt_render.i_visible_height *
-        i_aspect_num / i_aspect_den / p_vout->fmt_in.i_sar_num;
-    i_height = p_vout->fmt_render.i_visible_width * p_vout->fmt_in.i_sar_num *
-        i_aspect_den / i_aspect_num / p_vout->fmt_in.i_sar_den;
-
-    if( i_width < p_vout->fmt_render.i_visible_width )
+    if( !strcmp( psz_cmd, "crop" ) )
     {
-        p_vout->fmt_in.i_x_offset = p_vout->fmt_render.i_x_offset +
-            (p_vout->fmt_render.i_visible_width - i_width) / 2;
-        p_vout->fmt_in.i_visible_width = i_width;
+        char *psz_end = NULL, *psz_parser = strchr( newval.psz_string, ':' );
+        if( psz_parser )
+        {
+            /* We're using the 3:4 syntax */
+            i_aspect_num = strtol( newval.psz_string, &psz_end, 10 );
+            if( psz_end == newval.psz_string || !i_aspect_num ) goto crop_end;
+
+            i_aspect_den = strtol( ++psz_parser, &psz_end, 10 );
+            if( psz_end == psz_parser || !i_aspect_den ) goto crop_end;
+
+            i_width = p_vout->fmt_in.i_sar_den*p_vout->fmt_render.i_visible_height *
+                i_aspect_num / i_aspect_den / p_vout->fmt_in.i_sar_num;
+            i_height = p_vout->fmt_render.i_visible_width*p_vout->fmt_in.i_sar_num *
+                i_aspect_den / i_aspect_num / p_vout->fmt_in.i_sar_den;
+
+            if( i_width < p_vout->fmt_render.i_visible_width )
+            {
+                p_vout->fmt_in.i_x_offset = p_vout->fmt_render.i_x_offset +
+                    (p_vout->fmt_render.i_visible_width - i_width) / 2;
+                p_vout->fmt_in.i_visible_width = i_width;
+            }
+            else
+            {
+                p_vout->fmt_in.i_y_offset = p_vout->fmt_render.i_y_offset +
+                    (p_vout->fmt_render.i_visible_height - i_height) / 2;
+                p_vout->fmt_in.i_visible_height = i_height;
+            }
+        }
+        else
+        {
+            psz_parser = strchr( newval.psz_string, 'x' );
+            if( psz_parser )
+            {
+                /* Maybe we're using the <width>x<height>+<left>+<top> syntax */
+                unsigned int i_crop_width, i_crop_height, i_crop_top, i_crop_left;
+
+                i_crop_width = strtol( newval.psz_string, &psz_end, 10 );
+                if( psz_end != psz_parser ) goto crop_end;
+
+                psz_parser = strchr( ++psz_end, '+' );
+                i_crop_height = strtol( psz_end, &psz_end, 10 );
+                if( psz_end != psz_parser ) goto crop_end;
+
+                psz_parser = strchr( ++psz_end, '+' );
+                i_crop_left = strtol( psz_end, &psz_end, 10 );
+                if( psz_end != psz_parser ) goto crop_end;
+
+                psz_end++;
+                i_crop_top = strtol( psz_end, &psz_end, 10 );
+                if( *psz_end != '\0' ) goto crop_end;
+
+                i_width = i_crop_width;
+                p_vout->fmt_in.i_visible_width = i_width;
+
+                i_height = i_crop_height;
+                p_vout->fmt_in.i_visible_height = i_height;
+
+                p_vout->fmt_in.i_x_offset = i_crop_left;
+                p_vout->fmt_in.i_y_offset = i_crop_top;
+            }
+            else
+            {
+                /* Maybe we're using the <left>+<top>+<right>+<bottom> syntax */
+                unsigned int i_crop_top, i_crop_left, i_crop_bottom, i_crop_right;
+
+                psz_parser = strchr( newval.psz_string, '+' );
+                i_crop_left = strtol( newval.psz_string, &psz_end, 10 );
+                if( psz_end != psz_parser ) goto crop_end;
+
+                psz_parser = strchr( ++psz_end, '+' );
+                i_crop_top = strtol( psz_end, &psz_end, 10 );
+                if( psz_end != psz_parser ) goto crop_end;
+
+                psz_parser = strchr( ++psz_end, '+' );
+                i_crop_right = strtol( psz_end, &psz_end, 10 );
+                if( psz_end != psz_parser ) goto crop_end;
+
+                psz_end++;
+                i_crop_bottom = strtol( psz_end, &psz_end, 10 );
+                if( *psz_end != '\0' ) goto crop_end;
+
+                i_width = p_vout->fmt_render.i_visible_width
+                          - i_crop_left - i_crop_right;
+                p_vout->fmt_in.i_visible_width = i_width;
+
+                i_height = p_vout->fmt_render.i_visible_height
+                           - i_crop_top - i_crop_bottom;
+                p_vout->fmt_in.i_visible_height = i_height;
+
+                p_vout->fmt_in.i_x_offset = i_crop_left;
+                p_vout->fmt_in.i_y_offset = i_crop_top;
+            }
+        }
     }
-    else
+    else if( !strcmp( psz_cmd, "crop-top" )
+          || !strcmp( psz_cmd, "crop-left" )
+          || !strcmp( psz_cmd, "crop-bottom" )
+          || !strcmp( psz_cmd, "crop-right" ) )
     {
-        p_vout->fmt_in.i_y_offset = p_vout->fmt_render.i_y_offset +
-            (p_vout->fmt_render.i_visible_height - i_height) / 2;
+        unsigned int i_crop_top, i_crop_left, i_crop_bottom, i_crop_right;
+
+        i_crop_top = var_GetInteger( p_vout, "crop-top" );
+        i_crop_left = var_GetInteger( p_vout, "crop-left" );
+        i_crop_right = var_GetInteger( p_vout, "crop-right" );
+        i_crop_bottom = var_GetInteger( p_vout, "crop-bottom" );
+
+        i_width = p_vout->fmt_render.i_visible_width
+                  - i_crop_left - i_crop_right;
+        p_vout->fmt_in.i_visible_width = i_width;
+
+        i_height = p_vout->fmt_render.i_visible_height
+                   - i_crop_top - i_crop_bottom;
         p_vout->fmt_in.i_visible_height = i_height;
+
+        p_vout->fmt_in.i_x_offset = i_crop_left;
+        p_vout->fmt_in.i_y_offset = i_crop_top;
     }
 
  crop_end:
@@ -685,10 +1002,10 @@ static int AspectCallback( vlc_object_t *p_this, char const *psz_cmd,
 
     if( !psz_parser ) goto aspect_end;
 
-    i_aspect_num = strtol( newval.psz_string, &psz_end, 0 );
+    i_aspect_num = strtol( newval.psz_string, &psz_end, 10 );
     if( psz_end == newval.psz_string || !i_aspect_num ) goto aspect_end;
 
-    i_aspect_den = strtol( ++psz_parser, &psz_end, 0 );
+    i_aspect_den = strtol( ++psz_parser, &psz_end, 10 );
     if( psz_end == psz_parser || !i_aspect_den ) goto aspect_end;
 
     i_sar_num = i_aspect_num * p_vout->fmt_render.i_visible_height;
@@ -718,7 +1035,7 @@ static int AspectCallback( vlc_object_t *p_this, char const *psz_cmd,
              p_vout->fmt_in.i_sar_num, p_vout->fmt_in.i_sar_den );
 
     var_Get( p_vout, "crop", &val );
-    return CropCallback( p_this, 0, val, val, 0 );
+    return CropCallback( p_this, "crop", val, val, 0 );
 
     return VLC_SUCCESS;
 }
